@@ -7,11 +7,11 @@
     unused_mut,
     unused_variables,
     unused_unsafe,
-    unreachable_patterns,
-    unstable_features
+    unreachable_patterns
 )]
 #![doc(html_favicon_url = "https://wasmer.io/images/icons/favicon-32x32.png")]
 #![doc(html_logo_url = "https://github.com/wasmerio.png?size=200")]
+#![cfg_attr(docsrs, feature(doc_cfg, doc_auto_cfg))]
 
 #[macro_use]
 extern crate anyhow;
