@@ -3,6 +3,7 @@ use std::path::PathBuf;
 use anyhow::Context;
 use dialoguer::console::{style, Emoji};
 use indicatif::ProgressBar;
+use wasmer_package::utils::from_disk;
 
 /// Extract contents of a webc image to a directory.
 ///
@@ -32,10 +33,10 @@ pub struct PackageUnpack {
     ///   wasmer.toml and the full contents can not be restored.
     ///
     /// * webc
-    /// Directly unpack the webc contents.
-    /// - Volumes will be placed in subdirectories.
-    /// - atoms will be placed in the root directory
-    /// - the full webc manifest will be placed in a manifest.json file
+    ///   Directly unpack the webc contents.
+    ///   - Volumes will be placed in subdirectories.
+    ///   - atoms will be placed in the root directory
+    ///   - the full webc manifest will be placed in a manifest.json file
     #[clap(short, long, default_value = "package")]
     pub format: Format,
 }
@@ -67,7 +68,7 @@ impl PackageUnpack {
             PACKAGE_EMOJI
         ));
 
-        let pkg = webc::compat::Container::from_disk(&self.package_path).with_context(|| {
+        let pkg = from_disk(&self.package_path).with_context(|| {
             format!(
                 "could not open package at '{}'",
                 self.package_path.display()
