@@ -90,7 +90,7 @@ pub(crate) fn get_capability_cache_path(
                     n.name
                 ),
                 wasmer_config::package::PackageIdent::Hash(h) => {
-                    format!("hash_{}", h)
+                    format!("hash_{h}")
                 }
             },
             PackageSpecifier::Path(f) => {
