@@ -3,10 +3,13 @@ use std::path::PathBuf;
 
 use anyhow::{Context, Result};
 use clap::Parser;
-use wasmer::NativeEngineExt;
+use wasmer::{
+    sys::{engine::NativeEngineExt, Target},
+    Store,
+};
 use wasmer_wast::Wast as WastSpectest;
 
-use crate::{common::HashAlgorithm, store::StoreOptions};
+use crate::{backend::RuntimeOptions, common::HashAlgorithm};
 
 #[derive(Debug, Parser)]
 /// The options for the `wasmer wast` subcommand
@@ -16,7 +19,7 @@ pub struct Wast {
     path: PathBuf,
 
     #[clap(flatten)]
-    store: StoreOptions,
+    rt: RuntimeOptions,
 
     #[clap(short, long)]
     /// A flag to indicate wast stop at the first error or continue.
@@ -34,12 +37,12 @@ impl Wast {
             .context(format!("failed to test the wast `{}`", self.path.display()))
     }
     fn inner_execute(&self) -> Result<()> {
-        let (store, _compiler_name) = self.store.get_store()?;
+        let mut engine = self.rt.get_engine(&Target::default())?;
 
-        let mut engine = store.engine().clone();
         let hash_algorithm = self.hash_algorithm.unwrap_or_default().into();
         engine.set_hash_algorithm(Some(hash_algorithm));
 
+        let store: Store = Store::new(engine);
         let mut wast = WastSpectest::new_with_spectest(store);
         wast.fail_fast = self.fail_fast;
         wast.run_file(&self.path).with_context(|| "tests failed")?;
