@@ -10,6 +10,6 @@ mod unwind;
 pub use self::func_environ::{FuncEnvironment, GlobalVariable, TargetEnvironment};
 pub use self::func_translator::FuncTranslator;
 pub use self::translation_utils::{
-    irlibcall_to_libcall, irreloc_to_relocationkind, signature_to_cranelift_ir, type_to_irtype,
+    irlibcall_to_libcall, irreloc_to_relocationkind, signature_to_cranelift_ir,
 };
 pub(crate) use self::unwind::{compiled_function_unwind_info, CraneliftUnwindInfo};
