@@ -5,13 +5,14 @@ use inkwell::targets::{
 };
 pub use inkwell::OptimizationLevel as LLVMOptLevel;
 use itertools::Itertools;
-use std::fmt::Debug;
 use std::sync::Arc;
-use wasmer_compiler::{
-    types::target::{Architecture, OperatingSystem, Target, Triple},
-    Compiler, CompilerConfig, Engine, EngineBuilder, ModuleMiddleware,
+use std::{fmt::Debug, num::NonZero};
+use target_lexicon::BinaryFormat;
+use wasmer_compiler::{Compiler, CompilerConfig, Engine, EngineBuilder, ModuleMiddleware};
+use wasmer_types::{
+    target::{Architecture, OperatingSystem, Target, Triple},
+    Features, FunctionType, LocalFunctionIndex,
 };
-use wasmer_types::{FunctionType, LocalFunctionIndex};
 
 /// The InkWell ModuleInfo type
 pub type InkwellModule<'ctx> = inkwell::module::Module<'ctx>;
@@ -42,12 +43,16 @@ pub trait LLVMCallbacks: Debug + Send + Sync {
 #[derive(Debug, Clone)]
 pub struct LLVM {
     pub(crate) enable_nan_canonicalization: bool,
+    pub(crate) enable_g0m0_opt: bool,
     pub(crate) enable_verifier: bool,
+    pub(crate) enable_perfmap: bool,
     pub(crate) opt_level: LLVMOptLevel,
     is_pic: bool,
     pub(crate) callbacks: Option<Arc<dyn LLVMCallbacks>>,
     /// The middleware chain.
     pub(crate) middlewares: Vec<Arc<dyn ModuleMiddleware>>,
+    /// Number of threads to use when compiling a module.
+    pub(crate) num_threads: NonZero<usize>,
 }
 
 impl LLVM {
@@ -57,10 +62,13 @@ impl LLVM {
         Self {
             enable_nan_canonicalization: false,
             enable_verifier: false,
+            enable_perfmap: false,
             opt_level: LLVMOptLevel::Aggressive,
             is_pic: false,
             callbacks: None,
             middlewares: vec![],
+            enable_g0m0_opt: false,
+            num_threads: std::thread::available_parallelism().unwrap_or(NonZero::new(1).unwrap()),
         }
     }
 
@@ -70,6 +78,19 @@ impl LLVM {
         self
     }
 
+    /// (warning: experimental) Pass the value of the first (#0) global and the base pointer of the
+    /// first (#0) memory as parameter between guest functions.
+    pub fn enable_pass_params_opt(&mut self) -> &mut Self {
+        // internally, the "pass_params" opt is known as g0m0 opt.
+        self.enable_g0m0_opt = true;
+        self
+    }
+
+    pub fn num_threads(&mut self, num_threads: NonZero<usize>) -> &mut Self {
+        self.num_threads = num_threads;
+        self
+    }
+
     /// Callbacks that will triggered in the different compilation
     /// phases in LLVM.
     pub fn callbacks(&mut self, callbacks: Option<Arc<dyn LLVMCallbacks>>) -> &mut Self {
@@ -77,7 +98,11 @@ impl LLVM {
         self
     }
 
-    fn reloc_mode(&self) -> RelocMode {
+    fn reloc_mode(&self, binary_format: BinaryFormat) -> RelocMode {
+        if matches!(binary_format, BinaryFormat::Macho) {
+            return RelocMode::Static;
+        }
+
         if self.is_pic {
             RelocMode::PIC
         } else {
@@ -85,12 +110,16 @@ impl LLVM {
         }
     }
 
-    fn code_model(&self) -> CodeModel {
+    fn code_model(&self, binary_format: BinaryFormat) -> CodeModel {
         // We normally use the large code model, but when targeting shared
         // objects, we are required to use PIC. If we use PIC anyways, we lose
         // any benefit from large code model and there's some cost on all
         // platforms, plus some platforms (MachO) don't support PIC + large
         // at all.
+        if matches!(binary_format, BinaryFormat::Macho) {
+            return CodeModel::Default;
+        }
+
         if self.is_pic {
             CodeModel::Small
         } else {
@@ -98,6 +127,36 @@ impl LLVM {
         }
     }
 
+    pub(crate) fn target_operating_system(&self, target: &Target) -> OperatingSystem {
+        if target.triple().operating_system == OperatingSystem::Darwin && !self.is_pic {
+            // LLVM detects static relocation + darwin + 64-bit and
+            // force-enables PIC because MachO doesn't support that
+            // combination. They don't check whether they're targeting
+            // MachO, they check whether the OS is set to Darwin.
+            //
+            // Since both linux and darwin use SysV ABI, this should work.
+            //  but not in the case of Aarch64, there the ABI is slightly different
+            #[allow(clippy::match_single_binding)]
+            match target.triple().architecture {
+                Architecture::Aarch64(_) => OperatingSystem::Darwin,
+                _ => OperatingSystem::Linux,
+            }
+        } else {
+            target.triple().operating_system
+        }
+    }
+
+    pub(crate) fn target_binary_format(&self, target: &Target) -> target_lexicon::BinaryFormat {
+        if self.is_pic {
+            target.triple().binary_format
+        } else {
+            match self.target_operating_system(target) {
+                OperatingSystem::Darwin => target_lexicon::BinaryFormat::Macho,
+                _ => target_lexicon::BinaryFormat::Elf,
+            }
+        }
+    }
+
     fn target_triple(&self, target: &Target) -> TargetTriple {
         let architecture = if target.triple().architecture
             == Architecture::Riscv64(target_lexicon::Riscv64Architecture::Riscv64gc)
@@ -109,29 +168,9 @@ impl LLVM {
         // Hack: we're using is_pic to determine whether this is a native
         // build or not.
 
-        let operating_system =
-            if target.triple().operating_system == OperatingSystem::Darwin && !self.is_pic {
-                // LLVM detects static relocation + darwin + 64-bit and
-                // force-enables PIC because MachO doesn't support that
-                // combination. They don't check whether they're targeting
-                // MachO, they check whether the OS is set to Darwin.
-                //
-                // Since both linux and darwin use SysV ABI, this should work.
-                //  but not in the case of Aarch64, there the ABI is slightly different
-                #[allow(clippy::match_single_binding)]
-                match target.triple().architecture {
-                    Architecture::Aarch64(_) => OperatingSystem::Darwin,
-                    _ => OperatingSystem::Linux,
-                }
-            } else {
-                target.triple().operating_system
-            };
+        let operating_system = self.target_operating_system(target);
+        let binary_format = self.target_binary_format(target);
 
-        let binary_format = if self.is_pic {
-            target.triple().binary_format
-        } else {
-            target_lexicon::BinaryFormat::Elf
-        };
         let triple = Triple {
             architecture,
             vendor: target.triple().vendor.clone(),
@@ -200,7 +239,7 @@ impl LLVM {
         // are compliant with the same string representations as gcc.
         let llvm_cpu_features = cpu_features
             .iter()
-            .map(|feature| format!("+{}", feature))
+            .map(|feature| format!("+{feature}"))
             .join(",");
 
         let target_triple = self.target_triple(target);
@@ -219,10 +258,10 @@ impl LLVM {
                     _ => &llvm_cpu_features,
                 },
                 self.opt_level,
-                self.reloc_mode(),
+                self.reloc_mode(self.target_binary_format(target)),
                 match triple.architecture {
                     Architecture::LoongArch64 | Architecture::Riscv64(_) => CodeModel::Medium,
-                    _ => self.code_model(),
+                    _ => self.code_model(self.target_binary_format(target)),
                 },
             )
             .unwrap();
@@ -271,6 +310,10 @@ impl CompilerConfig for LLVM {
         self.is_pic = true;
     }
 
+    fn enable_perfmap(&mut self) {
+        self.enable_perfmap = true
+    }
+
     /// Whether to verify compiler IR.
     fn enable_verifier(&mut self) {
         self.enable_verifier = true;
@@ -289,6 +332,12 @@ impl CompilerConfig for LLVM {
     fn push_middleware(&mut self, middleware: Arc<dyn ModuleMiddleware>) {
         self.middlewares.push(middleware);
     }
+
+    fn supported_features_for_target(&self, _target: &Target) -> wasmer_types::Features {
+        let mut feats = Features::default();
+        feats.exceptions(true);
+        feats
+    }
 }
 
 impl Default for LLVM {
