@@ -5,9 +5,8 @@ use gimli::{
 use wasmer_compiler::types::{
     relocation::{Relocation, RelocationKind, RelocationTarget},
     section::{CustomSection, CustomSectionProtection, SectionBody},
-    target::Endianness,
 };
-use wasmer_types::{entity::EntityRef, LocalFunctionIndex};
+use wasmer_types::{entity::EntityRef, target::Endianness, LocalFunctionIndex};
 
 #[derive(Clone, Debug)]
 pub struct WriterRelocate {
@@ -36,6 +35,7 @@ impl WriterRelocate {
         let data = self.writer.into_vec();
         CustomSection {
             protection: CustomSectionProtection::Read,
+            alignment: None,
             bytes: SectionBody::new_with_vec(data),
             relocations: self.relocs,
         }
