@@ -3,35 +3,42 @@
 
 #[cfg(feature = "compiler")]
 use super::trampoline::{libcall_trampoline_len, make_libcall_trampolines};
-use crate::ArtifactCreate;
+
 #[cfg(feature = "compiler")]
-use crate::EngineInner;
-use crate::Features;
+use crate::{
+    serialize::SerializableCompilation, EngineInner, ModuleEnvironment, ModuleMiddlewareChain,
+};
+use crate::{
+    serialize::{
+        ArchivedSerializableCompilation, ArchivedSerializableModule, MetadataHeader,
+        SerializableModule,
+    },
+    types::{
+        function::{CompiledFunctionFrameInfo, FunctionBody, UnwindInfo, GOT},
+        module::CompileModuleInfo,
+        relocation::Relocation,
+        section::{CustomSection, SectionIndex},
+    },
+    ArtifactCreate, Features,
+};
 #[cfg(feature = "compiler")]
-use crate::{ModuleEnvironment, ModuleMiddlewareChain};
+use wasmer_types::target::Target;
+
 use core::mem::MaybeUninit;
 use enumset::EnumSet;
-use rkyv::de::deserializers::SharedDeserializeMap;
-use rkyv::option::ArchivedOption;
+use rkyv::rancor::Error as RkyvError;
 use self_cell::self_cell;
 use shared_buffer::OwnedBuffer;
 use std::sync::Arc;
-use wasmer_types::entity::{ArchivedPrimaryMap, PrimaryMap};
-use wasmer_types::ArchivedOwnedDataInitializer;
-use wasmer_types::ArchivedSerializableCompilation;
-use wasmer_types::ArchivedSerializableModule;
-use wasmer_types::CompileModuleInfo;
-use wasmer_types::DeserializeError;
 use wasmer_types::{
-    CompileError, CpuFeature, CustomSection, Dwarf, FunctionIndex, LocalFunctionIndex, MemoryIndex,
-    MemoryStyle, ModuleHash, ModuleInfo, OwnedDataInitializer, Relocation, SectionIndex,
-    SignatureIndex, TableIndex, TableStyle, Target,
+    entity::{ArchivedPrimaryMap, PrimaryMap},
+    target::CpuFeature,
+    DeserializeError,
 };
-use wasmer_types::{
-    CompiledFunctionFrameInfo, FunctionBody, HashAlgorithm, SerializableCompilation,
-    SerializableModule,
-};
-use wasmer_types::{MetadataHeader, SerializeError};
+
+// Not every compiler backend uses these.
+#[allow(unused)]
+use wasmer_types::*;
 
 /// A compiled wasm module, ready to be instantiated.
 #[cfg_attr(feature = "artifact-size", derive(loupe::MemoryUsage))]
@@ -135,9 +142,10 @@ impl ArtifactBuild {
             dynamic_function_trampolines: compilation.dynamic_function_trampolines,
             custom_sections,
             custom_section_relocations,
-            debug: compilation.debug,
+            unwind_info: compilation.unwind_info,
             libcall_trampolines,
             libcall_trampoline_len,
+            got: compilation.got,
         };
         let serializable = SerializableModule {
             compilation: serializable_compilation,
@@ -193,9 +201,14 @@ impl ArtifactBuild {
         self.serializable.compilation.libcall_trampoline_len as usize
     }
 
-    /// Get Debug optional Dwarf ref
-    pub fn get_debug_ref(&self) -> Option<&Dwarf> {
-        self.serializable.compilation.debug.as_ref()
+    /// Get a reference to the [`UnwindInfo`].
+    pub fn get_unwind_info(&self) -> &UnwindInfo {
+        &self.serializable.compilation.unwind_info
+    }
+
+    /// Get a reference to the [`GOT`].
+    pub fn get_got_ref(&self) -> &GOT {
+        &self.serializable.compilation.got
     }
 
     /// Get Function Relocations ref
@@ -271,7 +284,7 @@ impl<'a> ModuleFromArchive<'a> {
         Ok(Self {
             compilation: &module.compilation,
             data_initializers: &module.data_initializers,
-            cpu_features: module.cpu_features,
+            cpu_features: module.cpu_features.to_native(),
             original_module: module,
         })
     }
@@ -306,6 +319,7 @@ pub struct ArtifactBuildFromArchive {
 }
 
 impl ArtifactBuildFromArchive {
+    #[allow(unused)]
     pub(crate) fn try_new(
         buffer: OwnedBuffer,
         module_builder: impl FnOnce(
@@ -316,10 +330,9 @@ impl ArtifactBuildFromArchive {
 
         let cell = ArtifactBuildFromArchiveCell::try_new(buffer, |buffer| {
             let module = module_builder(buffer)?;
-            let mut deserializer = SharedDeserializeMap::new();
             compile_info = MaybeUninit::new(
-                rkyv::Deserialize::deserialize(&module.compile_info, &mut deserializer)
-                    .map_err(|e| DeserializeError::CorruptedBinary(format!("{:?}", e)))?,
+                rkyv::deserialize::<_, RkyvError>(&module.compile_info)
+                    .map_err(|e| DeserializeError::CorruptedBinary(format!("{e:?}")))?,
             );
             ModuleFromArchive::from_serializable_module(module)
         })?;
@@ -393,7 +406,10 @@ impl ArtifactBuildFromArchive {
 
     /// Get LibCall Trampoline Section Index
     pub fn get_libcall_trampolines(&self) -> SectionIndex {
-        self.cell.borrow_dependent().compilation.libcall_trampolines
+        rkyv::deserialize::<_, RkyvError>(
+            &self.cell.borrow_dependent().compilation.libcall_trampolines,
+        )
+        .unwrap()
     }
 
     /// Get LibCall Trampoline Length
@@ -401,15 +417,22 @@ impl ArtifactBuildFromArchive {
         self.cell
             .borrow_dependent()
             .compilation
-            .libcall_trampoline_len as usize
+            .libcall_trampoline_len
+            .to_native() as usize
     }
 
-    /// Get Debug optional Dwarf ref
-    pub fn get_debug_ref(&self) -> Option<&Dwarf> {
-        match self.cell.borrow_dependent().compilation.debug {
-            ArchivedOption::Some(ref x) => Some(x),
-            ArchivedOption::None => None,
-        }
+    /// Get an unarchived [`UnwindInfo`].
+    pub fn get_unwind_info(&self) -> UnwindInfo {
+        rkyv::deserialize::<_, rkyv::rancor::Error>(
+            &self.cell.borrow_dependent().compilation.unwind_info,
+        )
+        .unwrap()
+    }
+
+    /// Get an unarchived [`GOT`].
+    pub fn get_got_ref(&self) -> GOT {
+        rkyv::deserialize::<_, rkyv::rancor::Error>(&self.cell.borrow_dependent().compilation.got)
+            .unwrap()
     }
 
     /// Get Function Relocations ref
@@ -423,12 +446,10 @@ impl ArtifactBuildFromArchive {
     pub fn deserialize_frame_info_ref(
         &self,
     ) -> Result<PrimaryMap<LocalFunctionIndex, CompiledFunctionFrameInfo>, DeserializeError> {
-        let mut deserializer = SharedDeserializeMap::new();
-        rkyv::Deserialize::deserialize(
+        rkyv::deserialize::<_, RkyvError>(
             &self.cell.borrow_dependent().compilation.function_frame_info,
-            &mut deserializer,
         )
-        .map_err(|e| DeserializeError::CorruptedBinary(format!("{:?}", e)))
+        .map_err(|e| DeserializeError::CorruptedBinary(format!("{e:?}")))
     }
 }
 
@@ -479,12 +500,9 @@ impl<'a> ArtifactCreate<'a> for ArtifactBuildFromArchive {
         // deserialized from a file makes little sense, so hopefully, this is not a
         // common use-case.
 
-        let mut deserializer = SharedDeserializeMap::new();
-        let mut module: SerializableModule = rkyv::Deserialize::deserialize(
-            self.cell.borrow_dependent().original_module,
-            &mut deserializer,
-        )
-        .map_err(|e| SerializeError::Generic(e.to_string()))?;
+        let mut module: SerializableModule =
+            rkyv::deserialize::<_, RkyvError>(self.cell.borrow_dependent().original_module)
+                .map_err(|e| SerializeError::Generic(e.to_string()))?;
         module.compile_info = self.compile_info.clone();
         serialize_module(&module)
     }
