@@ -87,6 +87,18 @@ impl UnwindRegistry {
 
         Ok(())
     }
+
+    pub(crate) fn register_compact_unwind(
+        &mut self,
+        compact_unwind: Option<&[u8]>,
+        _eh_personality_addr_in_got: Option<usize>,
+    ) -> Result<(), String> {
+        if compact_unwind.is_some() {
+            return Err("Cannot register compact_unwind frames on Windows platforms".to_string());
+        }
+
+        Ok(())
+    }
 }
 
 impl Drop for UnwindRegistry {
