@@ -21,5 +21,7 @@ mod error;
 mod module;
 
 pub use self::error::ObjectError;
-pub use self::module::{emit_compilation, emit_data, emit_serialized, get_object_for_target};
+pub use self::module::{
+    emit_compilation, emit_data, emit_serialized, get_object_for_target, ObjectMetadataBuilder,
+};
 pub use object::{self, write::Object};
