@@ -1,20 +1,24 @@
 use super::error::ObjectError;
-use crate::types::{
-    function::Compilation,
-    relocation::{RelocationKind as Reloc, RelocationTarget},
-    section::{CustomSectionProtection, SectionIndex},
-    symbols::{Symbol, SymbolRegistry},
-    target::{Architecture, BinaryFormat, Endianness, Triple},
+use crate::{
+    serialize::MetadataHeader,
+    types::{
+        function::Compilation,
+        relocation::{RelocationKind as Reloc, RelocationTarget},
+        section::{CustomSectionProtection, SectionIndex},
+        symbols::{ModuleMetadata, Symbol, SymbolRegistry},
+    },
 };
 use object::{
     elf, macho,
     write::{
-        Object, Relocation, StandardSection, StandardSegment, Symbol as ObjSymbol, SymbolSection,
+        Object, Relocation, StandardSection, StandardSegment, Symbol as ObjSymbol, SymbolId,
+        SymbolSection,
     },
     FileFlags, RelocationEncoding, RelocationKind, SectionKind, SymbolFlags, SymbolKind,
     SymbolScope,
 };
-use wasmer_types::entity::PrimaryMap;
+use wasmer_types::entity::{EntityRef, PrimaryMap};
+use wasmer_types::target::{Architecture, BinaryFormat, Endianness, PointerWidth, Triple};
 use wasmer_types::LocalFunctionIndex;
 
 const DWARF_SECTION_NAME: &[u8] = b".eh_frame";
@@ -24,7 +28,7 @@ const DWARF_SECTION_NAME: &[u8] = b".eh_frame";
 /// # Usage
 ///
 /// ```rust
-/// # use wasmer_compiler::types::target::Triple;
+/// # use wasmer_types::target::Triple;
 /// # use wasmer_compiler::object::{ObjectError, get_object_for_target};
 ///
 /// # fn generate_object_for_target(triple: &Triple) -> Result<(), ObjectError> {
@@ -40,8 +44,7 @@ pub fn get_object_for_target(triple: &Triple) -> Result<Object, ObjectError> {
         BinaryFormat::Coff => object::BinaryFormat::Coff,
         binary_format => {
             return Err(ObjectError::UnsupportedBinaryFormat(format!(
-                "{}",
-                binary_format
+                "{binary_format}"
             )));
         }
     };
@@ -52,8 +55,7 @@ pub fn get_object_for_target(triple: &Triple) -> Result<Object, ObjectError> {
         Architecture::LoongArch64 => object::Architecture::LoongArch64,
         architecture => {
             return Err(ObjectError::UnsupportedArchitecture(format!(
-                "{}",
-                architecture
+                "{architecture}"
             )));
         }
     };
@@ -83,7 +85,7 @@ pub fn get_object_for_target(triple: &Triple) -> Result<Object, ObjectError> {
 /// # Usage
 ///
 /// ```rust
-/// # use wasmer_compiler::types::target::Triple;
+/// # use wasmer_types::target::Triple;
 /// # use wasmer_compiler::object::{ObjectError, get_object_for_target, emit_data};
 ///
 /// # fn emit_data_into_object(triple: &Triple) -> Result<(), ObjectError> {
@@ -98,7 +100,7 @@ pub fn emit_data(
     name: &[u8],
     data: &[u8],
     align: u64,
-) -> Result<(), ObjectError> {
+) -> Result<u64, ObjectError> {
     let symbol_id = obj.add_symbol(ObjSymbol {
         name: name.to_vec(),
         value: 0,
@@ -110,9 +112,9 @@ pub fn emit_data(
         flags: SymbolFlags::None,
     });
     let section_id = obj.section_id(StandardSection::Data);
-    obj.add_symbol_data(symbol_id, section_id, data, align);
+    let offset = obj.add_symbol_data(symbol_id, section_id, data, align);
 
-    Ok(())
+    Ok(offset)
 }
 
 /// Emit the compilation result into an existing object.
@@ -120,16 +122,18 @@ pub fn emit_data(
 /// # Usage
 ///
 /// ```rust
-/// # use wasmer_compiler::types::{ symbols::SymbolRegistry, target::{Triple}, function::{Compilation} };
-/// # use wasmer_compiler::object::{ObjectError, get_object_for_target, emit_compilation};
+/// # use wasmer_compiler::types::{ symbols::SymbolRegistry, function::{Compilation} };
+/// # use wasmer_types::target::Triple;
+/// # use wasmer_compiler::object::{ObjectError, ObjectMetadataBuilder, get_object_for_target, emit_compilation};
 ///
 /// # fn emit_compilation_into_object(
 /// #     triple: &Triple,
 /// #     compilation: Compilation,
+/// #     builder: ObjectMetadataBuilder,
 /// #     symbol_registry: impl SymbolRegistry,
 /// # ) -> Result<(), ObjectError> {
 /// let mut object = get_object_for_target(&triple)?;
-/// emit_compilation(&mut object, compilation, &symbol_registry, &triple)?;
+/// emit_compilation(&mut object, compilation, &symbol_registry, &triple, &builder)?;
 /// # Ok(())
 /// # }
 /// ```
@@ -138,6 +142,7 @@ pub fn emit_compilation(
     compilation: Compilation,
     symbol_registry: &impl SymbolRegistry,
     triple: &Triple,
+    relocs_builder: &ObjectMetadataBuilder,
 ) -> Result<(), ObjectError> {
     let mut function_bodies = PrimaryMap::with_capacity(compilation.functions.len());
     let mut function_relocations = PrimaryMap::with_capacity(compilation.functions.len());
@@ -151,12 +156,19 @@ pub fn emit_compilation(
         .map(|(_, section)| section.relocations.clone())
         .collect::<PrimaryMap<SectionIndex, _>>();
 
-    let debug_index = compilation.debug.map(|d| d.eh_frame);
+    let debug_index = compilation.unwind_info.eh_frame;
 
-    let align = match triple.architecture {
-        Architecture::X86_64 => 1,
-        // In Arm64 is recommended a 4-byte alignment
-        Architecture::Aarch64(_) => 4,
+    let default_align = match triple.architecture {
+        target_lexicon::Architecture::Aarch64(_) => {
+            if matches!(
+                triple.operating_system,
+                target_lexicon::OperatingSystem::Darwin
+            ) {
+                8
+            } else {
+                4
+            }
+        }
         _ => 1,
     };
 
@@ -170,7 +182,7 @@ pub fn emit_compilation(
                 let segment = obj.segment_name(StandardSegment::Debug).to_vec();
                 let section_id =
                     obj.add_section(segment, DWARF_SECTION_NAME.to_vec(), SectionKind::Debug);
-                obj.append_section_data(section_id, custom_section.bytes.as_slice(), align);
+                obj.append_section_data(section_id, custom_section.bytes.as_slice(), default_align);
                 let section_name = symbol_registry.symbol_to_name(Symbol::Section(section_index));
                 let symbol_id = obj.add_symbol(ObjSymbol {
                     name: section_name.into_bytes(),
@@ -206,7 +218,7 @@ pub fn emit_compilation(
                     symbol_id,
                     section_id,
                     custom_section.bytes.as_slice(),
-                    align,
+                    custom_section.alignment.unwrap_or(default_align),
                 );
                 (section_id, symbol_id)
             }
@@ -230,10 +242,13 @@ pub fn emit_compilation(
                 section: SymbolSection::Section(section_id),
                 flags: SymbolFlags::None,
             });
-            obj.add_symbol_data(symbol_id, section_id, &function.body, align);
+            obj.add_symbol_data(symbol_id, section_id, &function.body, default_align);
             (section_id, symbol_id)
         })
         .collect::<PrimaryMap<LocalFunctionIndex, _>>();
+    for (i, (_, symbol_id)) in function_symbol_ids.iter() {
+        relocs_builder.setup_function_pointer(obj, i.index(), *symbol_id)?;
+    }
 
     // Add function call trampolines
     for (signature_index, function) in compilation.function_call_trampolines.into_iter() {
@@ -250,7 +265,9 @@ pub fn emit_compilation(
             section: SymbolSection::Section(section_id),
             flags: SymbolFlags::None,
         });
-        obj.add_symbol_data(symbol_id, section_id, &function.body, align);
+        obj.add_symbol_data(symbol_id, section_id, &function.body, default_align);
+
+        relocs_builder.setup_trampoline(obj, signature_index.index(), symbol_id)?;
     }
 
     // Add dynamic function trampolines
@@ -268,7 +285,13 @@ pub fn emit_compilation(
             section: SymbolSection::Section(section_id),
             flags: SymbolFlags::None,
         });
-        obj.add_symbol_data(symbol_id, section_id, &function.body, align);
+        obj.add_symbol_data(symbol_id, section_id, &function.body, default_align);
+
+        relocs_builder.setup_dynamic_function_trampoline_pointer(
+            obj,
+            func_index.index(),
+            symbol_id,
+        )?;
     }
 
     let mut all_relocations = Vec::new();
@@ -314,7 +337,7 @@ pub fn emit_compilation(
                             value: macho::ARM64_RELOC_BRANCH26,
                             relative: true,
                         },
-                        fmt => panic!("unsupported binary format {:?}", fmt),
+                        fmt => panic!("unsupported binary format {fmt:?}"),
                     },
                     RelocationEncoding::Generic,
                     32,
@@ -324,9 +347,99 @@ pub fn emit_compilation(
                     RelocationEncoding::Generic,
                     32,
                 ),
+                Reloc::MachoArm64RelocBranch26 => (
+                    RelocationKind::MachO {
+                        value: macho::ARM64_RELOC_BRANCH26,
+                        relative: true,
+                    },
+                    RelocationEncoding::Generic,
+                    32,
+                ),
+                Reloc::MachoArm64RelocUnsigned => (
+                    RelocationKind::MachO {
+                        value: object::macho::ARM64_RELOC_UNSIGNED,
+                        relative: true,
+                    },
+                    RelocationEncoding::Generic,
+                    32,
+                ),
+                Reloc::MachoArm64RelocSubtractor => (
+                    RelocationKind::MachO {
+                        value: object::macho::ARM64_RELOC_SUBTRACTOR,
+                        relative: false,
+                    },
+                    RelocationEncoding::Generic,
+                    64,
+                ),
+                Reloc::MachoArm64RelocPage21 => (
+                    RelocationKind::MachO {
+                        value: object::macho::ARM64_RELOC_PAGE21,
+                        relative: true,
+                    },
+                    RelocationEncoding::Generic,
+                    32,
+                ),
+
+                Reloc::MachoArm64RelocPageoff12 => (
+                    RelocationKind::MachO {
+                        value: object::macho::ARM64_RELOC_PAGEOFF12,
+                        relative: false,
+                    },
+                    RelocationEncoding::Generic,
+                    32,
+                ),
+                Reloc::MachoArm64RelocGotLoadPage21 => (
+                    RelocationKind::MachO {
+                        value: object::macho::ARM64_RELOC_GOT_LOAD_PAGE21,
+                        relative: true,
+                    },
+                    RelocationEncoding::Generic,
+                    32,
+                ),
+                Reloc::MachoArm64RelocGotLoadPageoff12 => (
+                    RelocationKind::MachO {
+                        value: object::macho::ARM64_RELOC_GOT_LOAD_PAGEOFF12,
+                        relative: true,
+                    },
+                    RelocationEncoding::Generic,
+                    32,
+                ),
+                Reloc::MachoArm64RelocPointerToGot => (
+                    RelocationKind::MachO {
+                        value: object::macho::ARM64_RELOC_POINTER_TO_GOT,
+                        relative: true,
+                    },
+                    RelocationEncoding::Generic,
+                    32,
+                ),
+                Reloc::MachoArm64RelocTlvpLoadPage21 => (
+                    RelocationKind::MachO {
+                        value: object::macho::ARM64_RELOC_TLVP_LOAD_PAGE21,
+                        relative: true,
+                    },
+                    RelocationEncoding::Generic,
+                    32,
+                ),
+                Reloc::MachoArm64RelocTlvpLoadPageoff12 => (
+                    RelocationKind::MachO {
+                        value: object::macho::ARM64_RELOC_TLVP_LOAD_PAGEOFF12,
+                        relative: true,
+                    },
+                    RelocationEncoding::Generic,
+                    32,
+                ),
+                Reloc::MachoArm64RelocAddend => (
+                    RelocationKind::MachO {
+                        value: object::macho::ARM64_RELOC_ADDEND,
+                        relative: false,
+                    },
+                    RelocationEncoding::Generic,
+                    32,
+                ),
+
                 other => {
                     return Err(ObjectError::UnsupportedArchitecture(format!(
-                        "{} (relocation: {}",
+                        "{} (relocation: {})",
                         triple.architecture, other
                     )))
                 }
@@ -349,7 +462,13 @@ pub fn emit_compilation(
                     .map_err(ObjectError::Write)?;
                 }
                 RelocationTarget::LibCall(libcall) => {
-                    let libcall_fn_name = libcall.to_function_name().as_bytes();
+                    let mut libcall_fn_name = libcall.to_function_name().to_string();
+                    if matches!(triple.binary_format, BinaryFormat::Macho) {
+                        libcall_fn_name = format!("_{libcall_fn_name}");
+                    }
+
+                    let libcall_fn_name = libcall_fn_name.as_bytes();
+
                     // We add the symols lazily as we see them
                     let target_symbol = obj.symbol_id(libcall_fn_name).unwrap_or_else(|| {
                         obj.add_symbol(ObjSymbol {
@@ -403,7 +522,8 @@ pub fn emit_compilation(
 /// # Usage
 ///
 /// ```rust
-/// # use wasmer_compiler::types::{ symbols::SymbolRegistry, target::{Triple}, function::{Compilation} };
+/// # use wasmer_compiler::types::{ symbols::SymbolRegistry, function::{Compilation} };
+/// # use wasmer_types::target::Triple;
 /// # use wasmer_compiler::object::{ObjectError, get_object_for_target, emit_serialized};
 ///
 /// # fn emit_compilation_into_object(
@@ -425,8 +545,8 @@ pub fn emit_serialized(
 ) -> Result<(), ObjectError> {
     obj.set_mangling(object::write::Mangling::None);
     //let module_name = module.compile_info.module.name.clone();
-    let len_name = format!("{}_LENGTH", object_name);
-    let data_name = format!("{}_DATA", object_name);
+    let len_name = format!("{object_name}_LENGTH");
+    let data_name = format!("{object_name}_DATA");
     //let metadata_name = "WASMER_MODULE_METADATA";
 
     let align = match triple.architecture {
@@ -465,3 +585,194 @@ pub fn emit_serialized(
 
     Ok(())
 }
+
+/// ObjectMetadataBuilder builds serialized module metadata include in
+/// an object. In addition, it also relies on information from ModuleInfo
+/// to build a table of function pointers, trmampolines and dynamic function
+/// trampoline pointers. ObjectMetadataBuilder takes care of setting up
+/// relocations, so a linker can automatically fill in actuall addesses of
+/// all relavant functions. There is no need to piece the information together
+/// in the glue C file.
+pub struct ObjectMetadataBuilder {
+    placeholder_data: Vec<u8>,
+    metadata_length: u64,
+    section_offset: u64,
+    num_function_pointers: u64,
+    num_trampolines: u64,
+    num_dynamic_function_trampoline_pointers: u64,
+    endianness: Endianness,
+    pointer_width: PointerWidth,
+}
+
+impl ObjectMetadataBuilder {
+    /// Creates a new FunctionRelocsBuilder
+    pub fn new(metadata: &ModuleMetadata, triple: &Triple) -> Result<Self, ObjectError> {
+        let serialized_data = metadata.serialize()?;
+        let mut metadata_binary = vec![];
+        metadata_binary.extend(MetadataHeader::new(serialized_data.len()).into_bytes());
+        metadata_binary.extend(serialized_data);
+        let metadata_length = metadata_binary.len() as u64;
+
+        let pointer_width = triple.pointer_width().unwrap();
+        let endianness = triple
+            .endianness()
+            .map_err(|_| ObjectError::UnknownEndianness)?;
+
+        let module = &metadata.compile_info.module;
+        let num_function_pointers = module
+            .functions
+            .iter()
+            .filter(|(f_index, _)| module.local_func_index(*f_index).is_some())
+            .count() as u64;
+        let num_trampolines = module.signatures.len() as u64;
+        let num_dynamic_function_trampoline_pointers = module.num_imported_functions as u64;
+
+        let mut aself = Self {
+            placeholder_data: metadata_binary,
+            metadata_length,
+            section_offset: 0,
+            num_function_pointers,
+            num_trampolines,
+            num_dynamic_function_trampoline_pointers,
+            endianness,
+            pointer_width,
+        };
+
+        aself
+            .placeholder_data
+            .extend_from_slice(&aself.serialize_value(aself.num_function_pointers));
+        aself.placeholder_data.extend_from_slice(&vec![
+            0u8;
+            (aself.pointer_bytes() * aself.num_function_pointers)
+                as usize
+        ]);
+        aself
+            .placeholder_data
+            .extend_from_slice(&aself.serialize_value(aself.num_trampolines));
+        aself.placeholder_data.extend_from_slice(&vec![
+            0u8;
+            (aself.pointer_bytes() * aself.num_trampolines)
+                as usize
+        ]);
+        aself.placeholder_data.extend_from_slice(
+            &aself.serialize_value(aself.num_dynamic_function_trampoline_pointers),
+        );
+        aself.placeholder_data.extend_from_slice(&vec![
+            0u8;
+            (aself.pointer_bytes() * aself.num_dynamic_function_trampoline_pointers)
+                as usize
+        ]);
+
+        Ok(aself)
+    }
+
+    /// Sets section offset used in relocations
+    pub fn set_section_offset(&mut self, offset: u64) {
+        self.section_offset = offset;
+    }
+
+    /// Placeholder data for emit_data call
+    pub fn placeholder_data(&self) -> &[u8] {
+        &self.placeholder_data
+    }
+
+    /// Bytes of a pointer for target architecture
+    pub fn pointer_bytes(&self) -> u64 {
+        self.pointer_width.bytes() as u64
+    }
+
+    /// Sets up relocation for a function pointer
+    pub fn setup_function_pointer(
+        &self,
+        obj: &mut Object,
+        index: usize,
+        symbol_id: SymbolId,
+    ) -> Result<(), ObjectError> {
+        let section_id = obj.section_id(StandardSection::Data);
+        obj.add_relocation(
+            section_id,
+            Relocation {
+                offset: self.function_pointers_start_offset()
+                    + self.pointer_bytes() * (index as u64),
+                size: self.pointer_width.bits(),
+                kind: RelocationKind::Absolute,
+                encoding: RelocationEncoding::Generic,
+                symbol: symbol_id,
+                addend: 0,
+            },
+        )
+        .map_err(ObjectError::Write)
+    }
+
+    /// Sets up relocation for a trampoline
+    pub fn setup_trampoline(
+        &self,
+        obj: &mut Object,
+        index: usize,
+        symbol_id: SymbolId,
+    ) -> Result<(), ObjectError> {
+        let section_id = obj.section_id(StandardSection::Data);
+        obj.add_relocation(
+            section_id,
+            Relocation {
+                offset: self.trampolines_start_offset() + self.pointer_bytes() * (index as u64),
+                size: self.pointer_width.bits(),
+                kind: RelocationKind::Absolute,
+                encoding: RelocationEncoding::Generic,
+                symbol: symbol_id,
+                addend: 0,
+            },
+        )
+        .map_err(ObjectError::Write)
+    }
+
+    /// Sets up relocation for a dynamic function trampoline pointer
+    pub fn setup_dynamic_function_trampoline_pointer(
+        &self,
+        obj: &mut Object,
+        index: usize,
+        symbol_id: SymbolId,
+    ) -> Result<(), ObjectError> {
+        let section_id = obj.section_id(StandardSection::Data);
+        obj.add_relocation(
+            section_id,
+            Relocation {
+                offset: self.dynamic_function_trampoline_pointers_start_offset()
+                    + self.pointer_bytes() * (index as u64),
+                size: self.pointer_width.bits(),
+                kind: RelocationKind::Absolute,
+                encoding: RelocationEncoding::Generic,
+                symbol: symbol_id,
+                addend: 0,
+            },
+        )
+        .map_err(ObjectError::Write)
+    }
+
+    fn function_pointers_start_offset(&self) -> u64 {
+        self.section_offset + self.metadata_length + self.pointer_bytes()
+    }
+
+    fn trampolines_start_offset(&self) -> u64 {
+        self.function_pointers_start_offset()
+            + self.pointer_bytes() * self.num_function_pointers
+            + self.pointer_bytes()
+    }
+
+    fn dynamic_function_trampoline_pointers_start_offset(&self) -> u64 {
+        self.trampolines_start_offset()
+            + self.pointer_bytes() * self.num_trampolines
+            + self.pointer_bytes()
+    }
+
+    fn serialize_value(&self, value: u64) -> Vec<u8> {
+        match (self.endianness, self.pointer_width) {
+            (Endianness::Little, PointerWidth::U16) => (value as u16).to_le_bytes().to_vec(),
+            (Endianness::Big, PointerWidth::U16) => (value as u16).to_be_bytes().to_vec(),
+            (Endianness::Little, PointerWidth::U32) => (value as u32).to_le_bytes().to_vec(),
+            (Endianness::Big, PointerWidth::U32) => (value as u32).to_be_bytes().to_vec(),
+            (Endianness::Little, PointerWidth::U64) => value.to_le_bytes().to_vec(),
+            (Endianness::Big, PointerWidth::U64) => value.to_be_bytes().to_vec(),
+        }
+    }
+}
