@@ -1,6 +1,5 @@
 //! Generic Artifact abstraction for Wasmer Engines.
 
-use crate::types::target::CpuFeature;
 use crate::Features;
 use enumset::EnumSet;
 use std::any::Any;
@@ -8,7 +7,8 @@ use std::sync::Arc;
 use wasmer_types::entity::PrimaryMap;
 use wasmer_types::SerializeError;
 use wasmer_types::{
-    DataInitializerLike, MemoryIndex, MemoryStyle, ModuleInfo, TableIndex, TableStyle,
+    target::CpuFeature, DataInitializerLike, MemoryIndex, MemoryStyle, ModuleInfo, TableIndex,
+    TableStyle,
 };
 
 /// An `Artifact` is the product that the `Engine`
