@@ -1,6 +1,6 @@
 use std::path::Path;
 
-use wasmer_config::package::ModuleReference;
+use wasmer_config::package::{ModuleReference, SuggestedCompilerOptimizations, UserAnnotations};
 
 use webc::Container;
 
@@ -16,7 +16,7 @@ pub fn webc_to_package_dir(webc: &Container, target_dir: &Path) -> Result<(), Co
 
     let pkg_annotation = manifest
         .wapm()
-        .map_err(|err| ConversionError::with_cause("could not read package annotation", err))?;
+        .map_err(|err| ConversionError::msg(format!("could not read package annotation: {err}")))?;
     if let Some(ann) = pkg_annotation {
         let mut pkg = wasmer_config::package::Package::new_empty();
 
@@ -56,7 +56,7 @@ pub fn webc_to_package_dir(webc: &Container, target_dir: &Path) -> Result<(), Co
                 let (name, version) = if let Some((name, version_raw)) = raw.split_once('@') {
                     let version = version_raw.parse().map_err(|err| {
                         ConversionError::with_cause(
-                            format!("Could not parse version of dependency: '{}'", raw),
+                            format!("Could not parse version of dependency: '{raw}'"),
                             err,
                         )
                     })?;
@@ -74,7 +74,7 @@ pub fn webc_to_package_dir(webc: &Container, target_dir: &Path) -> Result<(), Co
 
     let fs_annotation = manifest
         .filesystem()
-        .map_err(|err| ConversionError::with_cause("could n ot read fs annotation", err))?;
+        .map_err(|err| ConversionError::msg(format!("could not read fs annotation: {err}")))?;
     if let Some(ann) = fs_annotation {
         for mapping in ann.0 {
             if mapping.from.is_some() {
@@ -134,7 +134,7 @@ pub fn webc_to_package_dir(webc: &Container, target_dir: &Path) -> Result<(), Co
     if !atoms.is_empty() {
         std::fs::create_dir_all(&module_dir).map_err(|err| {
             ConversionError::with_cause(
-                format!("Could not create directory '{}'", module_dir.display(),),
+                format!("Could not create directory '{}'", module_dir.display()),
                 err,
             )
         })?;
@@ -150,6 +150,29 @@ pub fn webc_to_package_dir(webc: &Container, target_dir: &Path) -> Result<(), Co
 
             let relative_path = format!("./{module_dir_name}/{atom_name}");
 
+            let mut annotations = None;
+
+            if let Some(manifest_atom) = manifest.atoms.get(&atom_name) {
+                if let Some(sco) = manifest_atom
+                    .annotations
+                    .get(SuggestedCompilerOptimizations::KEY)
+                {
+                    if let Some((_, v)) = sco.as_map().and_then(|v| {
+                        v.iter().find(|(k, _)| {
+                            k.as_text().is_some_and(|v| {
+                                v == SuggestedCompilerOptimizations::PASS_PARAMS_KEY
+                            })
+                        })
+                    }) {
+                        annotations = Some(UserAnnotations {
+                            suggested_compiler_optimizations: SuggestedCompilerOptimizations {
+                                pass_params: Some(v.as_bool().unwrap_or_default()),
+                            },
+                        });
+                    }
+                }
+            }
+
             pkg_manifest.modules.push(wasmer_config::package::Module {
                 name: atom_name,
                 source: relative_path.into(),
@@ -157,6 +180,7 @@ pub fn webc_to_package_dir(webc: &Container, target_dir: &Path) -> Result<(), Co
                 kind: None,
                 interfaces: None,
                 bindings: None,
+                annotations,
             });
         }
     }
@@ -177,10 +201,9 @@ pub fn webc_to_package_dir(webc: &Container, target_dir: &Path) -> Result<(), Co
         let atom_annotation = spec
             .annotation::<webc::metadata::annotations::Atom>(webc::metadata::annotations::Atom::KEY)
             .map_err(|err| {
-                ConversionError::with_cause(
-                    format!("could not read atom annotation for command '{}'", name),
-                    err,
-                )
+                ConversionError::msg(format!(
+                    "could not read atom annotation for command '{name}': {err}"
+                ))
             })?
             .ok_or_else(|| {
                 ConversionError::msg(format!(
