@@ -36,6 +36,8 @@ pub enum Type {
     ExternRef, /* = 128 */
     /// A reference to a Wasm function.
     FuncRef,
+    /// A reference to a Wasm exception.
+    ExceptionRef,
 }
 
 impl Type {
@@ -50,13 +52,13 @@ impl Type {
 
     /// Returns true if `Type` matches either of the reference types.
     pub fn is_ref(self) -> bool {
-        matches!(self, Self::ExternRef | Self::FuncRef)
+        matches!(self, Self::ExternRef | Self::FuncRef | Self::ExceptionRef)
     }
 }
 
 impl fmt::Display for Type {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        write!(f, "{:?}", self)
+        write!(f, "{self:?}")
     }
 }
 
@@ -128,6 +130,8 @@ pub enum ExternType {
     Table(TableType),
     /// This external type is the type of a WebAssembly memory.
     Memory(MemoryType),
+    /// This external type is the type of a WebAssembly tag.
+    Tag(TagType),
 }
 
 fn is_global_compatible(exported: GlobalType, imported: GlobalType) -> bool {
@@ -234,6 +238,7 @@ impl ExternType {
             (Self::Global(a), Self::Global(b)) => is_global_compatible(*a, *b),
             (Self::Table(a), Self::Table(b)) => is_table_compatible(a, b, runtime_size),
             (Self::Memory(a), Self::Memory(b)) => is_memory_compatible(a, b, runtime_size),
+            (Self::Tag(a), Self::Tag(b)) => a == b,
             // The rest of possibilities, are not compatible
             _ => false,
         }
@@ -287,16 +292,16 @@ impl fmt::Display for FunctionType {
         let params = self
             .params
             .iter()
-            .map(|p| format!("{:?}", p))
+            .map(|p| format!("{p:?}"))
             .collect::<Vec<_>>()
             .join(", ");
         let results = self
             .results
             .iter()
-            .map(|p| format!("{:?}", p))
+            .map(|p| format!("{p:?}"))
             .collect::<Vec<_>>()
             .join(", ");
-        write!(f, "[{}] -> [{}]", params, results)
+        write!(f, "[{params}] -> [{results}]")
     }
 }
 
@@ -445,6 +450,67 @@ pub enum GlobalInit {
     RefFunc(FunctionIndex),
 }
 
+// Tag Types
+
+/// The kind of a [`Tag`].
+///
+/// Currently, tags can only express exceptions.
+#[derive(Debug, Clone, PartialEq, Eq, Hash)]
+#[cfg_attr(feature = "enable-serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "artifact-size", derive(loupe::MemoryUsage))]
+#[derive(RkyvSerialize, RkyvDeserialize, Archive)]
+#[rkyv(derive(Debug))]
+pub enum TagKind {
+    /// This tag's event is an exception.
+    Exception,
+}
+
+/// The signature of a tag that is either implemented
+/// in a Wasm module or exposed to Wasm by the host.
+#[derive(Debug, Clone, PartialEq, Eq, Hash)]
+#[cfg_attr(feature = "enable-serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "artifact-size", derive(loupe::MemoryUsage))]
+#[derive(RkyvSerialize, RkyvDeserialize, Archive)]
+#[rkyv(derive(Debug))]
+pub struct TagType {
+    /// The kind of the tag.
+    pub kind: TagKind,
+    /// The parameters of the function
+    pub params: Box<[Type]>,
+}
+
+impl TagType {
+    /// Creates a new [`TagType`] with the given kind, parameter and return types.
+    pub fn new<Params>(kind: TagKind, params: Params) -> Self
+    where
+        Params: Into<Box<[Type]>>,
+    {
+        Self {
+            kind,
+            params: params.into(),
+        }
+    }
+
+    /// Parameter types.
+    pub fn params(&self) -> &[Type] {
+        &self.params
+    }
+
+    /// Create a new [`TagType`] with the given kind and the associated type.
+    pub fn from_fn_type(kind: TagKind, ty: FunctionType) -> Self {
+        Self {
+            kind,
+            params: ty.params().into(),
+        }
+    }
+}
+
+impl fmt::Display for TagType {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        write!(f, "({:?}) {:?}", self.kind, self.params(),)
+    }
+}
+
 // Table Types
 
 /// A descriptor for a table in a WebAssembly module.
