@@ -67,6 +67,15 @@ impl FileSystem for EmptyFileSystem {
     fn new_open_options(&self) -> OpenOptions {
         OpenOptions::new(self)
     }
+
+    fn mount(
+        &self,
+        name: String,
+        path: &Path,
+        fs: Box<dyn crate::FileSystem + Send + Sync>,
+    ) -> Result<()> {
+        Err(FsError::Unsupported)
+    }
 }
 
 impl FileOpener for EmptyFileSystem {
