@@ -8,7 +8,7 @@ use crate::table::VMTable;
 use crate::vmcontext::VMFunctionKind;
 use crate::{MaybeInstanceOwned, VMCallerCheckedAnyfunc};
 use std::any::Any;
-use wasmer_types::FunctionType;
+use wasmer_types::{FunctionType, TagKind};
 
 /// The value of an export passed from one instance to another.
 #[cfg_attr(feature = "artifact-size", derive(loupe::MemoryUsage))]
@@ -24,6 +24,9 @@ pub enum VMExtern {
 
     /// A global export value.
     Global(InternalStoreHandle<VMGlobal>),
+
+    /// A tag export value.
+    Tag(InternalStoreHandle<VMTag>),
 }
 
 /// A function export value.
@@ -43,3 +46,20 @@ pub struct VMFunction {
     /// Associated data owned by a host function.
     pub host_data: Box<dyn Any>,
 }
+
+/// A tag export value.
+#[derive(Debug, Clone, Eq, PartialEq)]
+pub struct VMTag {
+    /// The kind of tag.
+    // Note: currently it can only be exception.
+    pub kind: TagKind,
+    /// The tag type, used for compatibility checking.
+    pub signature: FunctionType,
+}
+
+impl VMTag {
+    /// Create a new [`VMTag`].
+    pub fn new(kind: TagKind, signature: FunctionType) -> Self {
+        Self { kind, signature }
+    }
+}
