@@ -2,8 +2,9 @@
 // Attributions: https://github.com/wasmerio/wasmer/blob/main/docs/ATTRIBUTIONS.md
 
 use crate::vmcontext::{VMFunctionImport, VMGlobalImport, VMMemoryImport, VMTableImport};
+use crate::VMTagImport;
 use wasmer_types::entity::{BoxedSlice, PrimaryMap};
-use wasmer_types::{FunctionIndex, GlobalIndex, MemoryIndex, TableIndex};
+use wasmer_types::{FunctionIndex, GlobalIndex, MemoryIndex, TableIndex, TagIndex};
 
 /// Resolved import pointers.
 #[derive(Clone)]
@@ -17,6 +18,9 @@ pub struct Imports {
     /// Resolved addresses for imported memories.
     pub memories: BoxedSlice<MemoryIndex, VMMemoryImport>,
 
+    /// Resolved addresses for imported memories.
+    pub tags: BoxedSlice<TagIndex, VMTagImport>,
+
     /// Resolved addresses for imported globals.
     pub globals: BoxedSlice<GlobalIndex, VMGlobalImport>,
 }
@@ -27,12 +31,14 @@ impl Imports {
         function_imports: PrimaryMap<FunctionIndex, VMFunctionImport>,
         table_imports: PrimaryMap<TableIndex, VMTableImport>,
         memory_imports: PrimaryMap<MemoryIndex, VMMemoryImport>,
+        tag_imports: PrimaryMap<TagIndex, VMTagImport>,
         global_imports: PrimaryMap<GlobalIndex, VMGlobalImport>,
     ) -> Self {
         Self {
             functions: function_imports.into_boxed_slice(),
             tables: table_imports.into_boxed_slice(),
             memories: memory_imports.into_boxed_slice(),
+            tags: tag_imports.into_boxed_slice(),
             globals: global_imports.into_boxed_slice(),
         }
     }
@@ -44,6 +50,7 @@ impl Imports {
             tables: PrimaryMap::new().into_boxed_slice(),
             memories: PrimaryMap::new().into_boxed_slice(),
             globals: PrimaryMap::new().into_boxed_slice(),
+            tags: PrimaryMap::new().into_boxed_slice(),
         }
     }
 }
