@@ -17,7 +17,7 @@ use std::{
     path::{Component, Path, PathBuf},
     pin::Pin,
     sync::{
-        atomic::{AtomicBool, AtomicU64, Ordering},
+        atomic::{AtomicBool, AtomicI32, AtomicU64, Ordering},
         Arc, Mutex, RwLock, Weak,
     },
     task::{Context, Poll},
@@ -31,19 +31,19 @@ use crate::{
 use futures::{future::BoxFuture, Future, TryStreamExt};
 #[cfg(feature = "enable-serde")]
 use serde_derive::{Deserialize, Serialize};
-use tokio::{io::AsyncWriteExt, runtime::Handle};
+use tokio::io::AsyncWriteExt;
 use tracing::{debug, trace};
 use virtual_fs::{copy_reference, FileSystem, FsError, OpenOptions, VirtualFile};
 use wasmer_config::package::PackageId;
 use wasmer_wasix_types::{
     types::{__WASI_STDERR_FILENO, __WASI_STDIN_FILENO, __WASI_STDOUT_FILENO},
     wasi::{
-        Errno, Fd as WasiFd, Fdflags, Fdstat, Filesize, Filestat, Filetype, Preopentype, Prestat,
-        PrestatEnum, Rights, Socktype,
+        Errno, Fd as WasiFd, Fdflags, Fdflagsext, Fdstat, Filesize, Filestat, Filetype,
+        Preopentype, Prestat, PrestatEnum, Rights, Socktype,
     },
 };
 
-pub use self::fd::{EpollFd, EpollInterest, EpollJoinGuard, Fd, InodeVal, Kind};
+pub use self::fd::{EpollFd, EpollInterest, EpollJoinGuard, Fd, FdInner, InodeVal, Kind};
 pub(crate) use self::inode_guard::{
     InodeValFilePollGuard, InodeValFilePollGuardJoin, InodeValFilePollGuardMode,
     InodeValFileReadGuard, InodeValFileWriteGuard, WasiStateFileGuard, POLL_GUARD_MAX_RET,
@@ -83,6 +83,7 @@ const STDIN_DEFAULT_RIGHTS: Rights = {
             | Rights::FD_SYNC.bits()
             | Rights::FD_ADVISE.bits()
             | Rights::FD_FILESTAT_GET.bits()
+            | Rights::FD_FDSTAT_SET_FLAGS.bits()
             | Rights::POLL_FD_READWRITE.bits(),
     )
 };
@@ -95,6 +96,7 @@ const STDOUT_DEFAULT_RIGHTS: Rights = {
             | Rights::FD_WRITE.bits()
             | Rights::FD_ADVISE.bits()
             | Rights::FD_FILESTAT_GET.bits()
+            | Rights::FD_FDSTAT_SET_FLAGS.bits()
             | Rights::POLL_FD_READWRITE.bits(),
     )
 };
@@ -105,6 +107,7 @@ const STDERR_DEFAULT_RIGHTS: Rights = STDOUT_DEFAULT_RIGHTS;
 pub const MAX_SYMLINKS: u32 = 128;
 
 #[derive(Debug, Clone, Copy, PartialEq, Eq, PartialOrd, Ord, Hash)]
+#[cfg_attr(feature = "enable-serde", derive(Serialize, Deserialize))]
 pub struct Inode(u64);
 
 impl Inode {
@@ -121,20 +124,85 @@ impl Inode {
 pub struct InodeGuard {
     ino: Inode,
     inner: Arc<InodeVal>,
+
+    // This exists because self.inner doesn't really represent the
+    // number of FDs referencing this InodeGuard. We need that number
+    // so we can know when to drop the file handle, which should result
+    // in the backing file (which may be a host file) getting closed.
+    open_handles: Arc<AtomicI32>,
 }
 impl InodeGuard {
     pub fn ino(&self) -> Inode {
         self.ino
     }
+
     pub fn downgrade(&self) -> InodeWeakGuard {
         InodeWeakGuard {
             ino: self.ino,
+            open_handles: self.open_handles.clone(),
             inner: Arc::downgrade(&self.inner),
         }
     }
+
     pub fn ref_cnt(&self) -> usize {
         Arc::strong_count(&self.inner)
     }
+
+    pub fn handle_count(&self) -> u32 {
+        self.open_handles.load(Ordering::SeqCst) as u32
+    }
+
+    pub fn acquire_handle(&self) {
+        let prev_handles = self.open_handles.fetch_add(1, Ordering::SeqCst);
+        trace!(ino = %self.ino.0, new_count = %(prev_handles + 1), "acquiring handle for InodeGuard");
+    }
+
+    pub fn drop_one_handle(&self) {
+        let prev_handles = self.open_handles.fetch_sub(1, Ordering::SeqCst);
+
+        trace!(ino = %self.ino.0, %prev_handles, "dropping handle for InodeGuard");
+
+        // If this wasn't the last handle, nothing else to do...
+        if prev_handles > 1 {
+            return;
+        }
+
+        // ... otherwise, drop the VirtualFile reference
+        let mut guard = self.inner.write();
+
+        // Must have at least one open handle before we can drop.
+        // This check happens after `inner` is locked so we can
+        // poison the lock and keep people from using this (possibly
+        // corrupt) InodeGuard.
+        if prev_handles != 1 {
+            panic!("InodeGuard handle dropped too many times");
+        }
+
+        // Re-check the open handles to account for race conditions
+        if self.open_handles.load(Ordering::SeqCst) != 0 {
+            return;
+        }
+
+        let ino = self.ino.0;
+        trace!(%ino, "InodeGuard has no more open handles");
+
+        match guard.deref_mut() {
+            Kind::File { handle, .. } if handle.is_some() => {
+                let file_ref_count = Arc::strong_count(handle.as_ref().unwrap());
+                trace!(%file_ref_count, %ino, "dropping file handle");
+                drop(handle.take().unwrap());
+            }
+            Kind::PipeRx { rx } => {
+                trace!(%ino, "closing pipe rx");
+                rx.close();
+            }
+            Kind::PipeTx { tx } => {
+                trace!(%ino, "closing pipe tx");
+                tx.close();
+            }
+            _ => (),
+        }
+    }
 }
 impl std::ops::Deref for InodeGuard {
     type Target = InodeVal;
@@ -146,6 +214,10 @@ impl std::ops::Deref for InodeGuard {
 #[derive(Debug, Clone)]
 pub struct InodeWeakGuard {
     ino: Inode,
+    // Needed for when we want to upgrade back. We don't exactly
+    // care too much when the AtomicI32 is dropped, so this is
+    // a strong reference to keep things simple.
+    open_handles: Arc<AtomicI32>,
     inner: Weak<InodeVal>,
 }
 impl InodeWeakGuard {
@@ -155,6 +227,7 @@ impl InodeWeakGuard {
     pub fn upgrade(&self) -> Option<InodeGuard> {
         Weak::upgrade(&self.inner).map(|inner| InodeGuard {
             ino: self.ino,
+            open_handles: self.open_handles.clone(),
             inner,
         })
     }
@@ -198,7 +271,13 @@ impl WasiInodes {
             guard.lookup.retain(|_, v| Weak::strong_count(v) > 0);
         }
 
-        InodeGuard { ino, inner: val }
+        let open_handles = Arc::new(AtomicI32::new(0));
+
+        InodeGuard {
+            ino,
+            open_handles,
+            inner: val,
+        }
     }
 
     /// Get the `VirtualFile` object at stdout
@@ -495,7 +574,42 @@ impl WasiFs {
     }
 
     /// Closes all the file handles.
-    #[allow(clippy::await_holding_lock)]
+    pub async fn close_cloexec_fds(&self) {
+        let to_close = {
+            if let Ok(map) = self.fd_map.read() {
+                map.iter()
+                    .filter_map(|(k, v)| {
+                        if v.inner.fd_flags.contains(Fdflagsext::CLOEXEC)
+                            && !v.is_stdio
+                            && !v.inode.is_preopened
+                        {
+                            tracing::trace!(fd = %k, "Closing FD due to CLOEXEC flag");
+                            Some(k)
+                        } else {
+                            None
+                        }
+                    })
+                    .collect::<HashSet<_>>()
+            } else {
+                HashSet::new()
+            }
+        };
+
+        let _ = tokio::join!(async {
+            for fd in &to_close {
+                self.flush(*fd).await.ok();
+                self.close_fd(*fd).ok();
+            }
+        });
+
+        if let Ok(mut map) = self.fd_map.write() {
+            for fd in &to_close {
+                map.remove(*fd);
+            }
+        }
+    }
+
+    /// Closes all the file handles.
     pub async fn close_all(&self) {
         let mut to_close = {
             if let Ok(map) = self.fd_map.read() {
@@ -615,6 +729,7 @@ impl WasiFs {
     ///   unlikely in pratice.  [Join the discussion](https://github.com/wasmerio/wasmer/issues/1219)
     ///   for what the newer, safer WASI FS APIs should look like.
     #[allow(dead_code)]
+    #[allow(clippy::too_many_arguments)]
     pub unsafe fn open_dir_all(
         &mut self,
         inodes: &WasiInodes,
@@ -623,6 +738,7 @@ impl WasiFs {
         rights: Rights,
         rights_inheriting: Rights,
         flags: Fdflags,
+        fd_flags: Fdflagsext,
     ) -> Result<WasiFd, FsError> {
         // TODO: check permissions here? probably not, but this should be
         // an explicit choice, so justify it in a comment when we remove this one
@@ -678,6 +794,7 @@ impl WasiFs {
             rights,
             rights_inheriting,
             flags,
+            fd_flags,
             Fd::READ | Fd::WRITE,
             cur_inode,
         )
@@ -687,7 +804,7 @@ impl WasiFs {
     /// Opens a user-supplied file in the directory specified with the
     /// name and flags given
     // dead code because this is an API for external use
-    // TODO: is this used anywhere?
+    // TODO: is this used anywhere? Is it even sound?
     #[allow(dead_code, clippy::too_many_arguments)]
     pub fn open_file_at(
         &mut self,
@@ -699,6 +816,7 @@ impl WasiFs {
         rights: Rights,
         rights_inheriting: Rights,
         flags: Fdflags,
+        fd_flags: Fdflagsext,
     ) -> Result<WasiFd, FsError> {
         // TODO: check permissions here? probably not, but this should be
         // an explicit choice, so justify it in a comment when we remove this one
@@ -738,7 +856,14 @@ impl WasiFs {
 
                 // Here, we clone the inode so we can use it to overwrite the fd field below.
                 let real_fd = self
-                    .create_fd(rights, rights_inheriting, flags, open_flags, inode.clone())
+                    .create_fd(
+                        rights,
+                        rights_inheriting,
+                        flags,
+                        fd_flags,
+                        open_flags,
+                        inode.clone(),
+                    )
                     .map_err(fs_error_from_wasi_err)?;
 
                 {
@@ -1105,7 +1230,9 @@ impl WasiFs {
                     }
                     Kind::File { .. }
                     | Kind::Socket { .. }
-                    | Kind::Pipe { .. }
+                    | Kind::PipeRx { .. }
+                    | Kind::PipeTx { .. }
+                    | Kind::DuplexPipe { .. }
                     | Kind::EventNotifications { .. }
                     | Kind::Epoll { .. } => {
                         return Err(Errno::Notdir);
@@ -1307,10 +1434,13 @@ impl WasiFs {
 
         if ret.is_err() && fd == VIRTUAL_ROOT_FD {
             Ok(Fd {
-                rights: ALL_RIGHTS,
-                rights_inheriting: ALL_RIGHTS,
-                flags: Fdflags::empty(),
-                offset: Arc::new(AtomicU64::new(0)),
+                inner: FdInner {
+                    rights: ALL_RIGHTS,
+                    rights_inheriting: ALL_RIGHTS,
+                    flags: Fdflags::empty(),
+                    offset: Arc::new(AtomicU64::new(0)),
+                    fd_flags: Fdflagsext::empty(),
+                },
                 open_flags: 0,
                 inode: self.root_inode.clone(),
                 is_stdio: false,
@@ -1399,9 +1529,9 @@ impl WasiFs {
                 },
                 _ => Filetype::Unknown,
             },
-            fs_flags: fd.flags,
-            fs_rights_base: fd.rights,
-            fs_rights_inheriting: fd.rights_inheriting, // TODO(lachlan): Is this right?
+            fs_flags: fd.inner.flags,
+            fs_rights_base: fd.inner.rights,
+            fs_rights_inheriting: fd.inner.rights_inheriting, // TODO(lachlan): Is this right?
         })
     }
 
@@ -1445,7 +1575,7 @@ impl WasiFs {
             }
             _ => {
                 let fd = self.get_fd(fd)?;
-                if !fd.rights.contains(Rights::FD_DATASYNC) {
+                if !fd.inner.rights.contains(Rights::FD_DATASYNC) {
                     return Err(Errno::Access);
                 }
 
@@ -1543,14 +1673,16 @@ impl WasiFs {
         &self,
         rights: Rights,
         rights_inheriting: Rights,
-        flags: Fdflags,
+        fs_flags: Fdflags,
+        fd_flags: Fdflagsext,
         open_flags: u16,
         inode: InodeGuard,
     ) -> Result<WasiFd, Errno> {
         self.create_fd_ext(
             rights,
             rights_inheriting,
-            flags,
+            fs_flags,
+            fd_flags,
             open_flags,
             inode,
             None,
@@ -1558,11 +1690,13 @@ impl WasiFs {
         )
     }
 
+    #[allow(clippy::too_many_arguments)]
     pub fn with_fd(
         &self,
         rights: Rights,
         rights_inheriting: Rights,
-        flags: Fdflags,
+        fs_flags: Fdflags,
+        fd_flags: Fdflagsext,
         open_flags: u16,
         inode: InodeGuard,
         idx: WasiFd,
@@ -1570,7 +1704,8 @@ impl WasiFs {
         self.create_fd_ext(
             rights,
             rights_inheriting,
-            flags,
+            fs_flags,
+            fd_flags,
             open_flags,
             inode,
             Some(idx),
@@ -1584,7 +1719,8 @@ impl WasiFs {
         &self,
         rights: Rights,
         rights_inheriting: Rights,
-        flags: Fdflags,
+        fs_flags: Fdflags,
+        fd_flags: Fdflagsext,
         open_flags: u16,
         inode: InodeGuard,
         idx: Option<WasiFd>,
@@ -1595,10 +1731,13 @@ impl WasiFs {
             Some(__WASI_STDIN_FILENO) | Some(__WASI_STDOUT_FILENO) | Some(__WASI_STDERR_FILENO)
         );
         let fd = Fd {
-            rights,
-            rights_inheriting,
-            flags,
-            offset: Arc::new(AtomicU64::new(0)),
+            inner: FdInner {
+                rights,
+                rights_inheriting,
+                flags: fs_flags,
+                offset: Arc::new(AtomicU64::new(0)),
+                fd_flags,
+            },
             open_flags,
             inode,
             is_stdio,
@@ -1619,16 +1758,38 @@ impl WasiFs {
     }
 
     pub fn clone_fd(&self, fd: WasiFd) -> Result<WasiFd, Errno> {
+        self.clone_fd_ext(fd, 0, None)
+    }
+
+    pub fn clone_fd_ext(
+        &self,
+        fd: WasiFd,
+        min_result_fd: WasiFd,
+        cloexec: Option<bool>,
+    ) -> Result<WasiFd, Errno> {
         let fd = self.get_fd(fd)?;
-        Ok(self.fd_map.write().unwrap().insert_first_free(Fd {
-            rights: fd.rights,
-            rights_inheriting: fd.rights_inheriting,
-            flags: fd.flags,
-            offset: fd.offset.clone(),
-            open_flags: fd.open_flags,
-            inode: fd.inode,
-            is_stdio: fd.is_stdio,
-        }))
+        Ok(self.fd_map.write().unwrap().insert_first_free_after(
+            Fd {
+                inner: FdInner {
+                    rights: fd.inner.rights,
+                    rights_inheriting: fd.inner.rights_inheriting,
+                    flags: fd.inner.flags,
+                    offset: fd.inner.offset.clone(),
+                    fd_flags: match cloexec {
+                        None => fd.inner.fd_flags,
+                        Some(cloexec) => {
+                            let mut f = fd.inner.fd_flags;
+                            f.set(Fdflagsext::CLOEXEC, cloexec);
+                            f
+                        }
+                    },
+                },
+                open_flags: fd.open_flags,
+                inode: fd.inode,
+                is_stdio: fd.is_stdio,
+            },
+            min_result_fd,
+        ))
     }
 
     /// Low level function to remove an inode, that is it deletes the WASI FS's
@@ -1707,10 +1868,11 @@ impl WasiFs {
                 root_rights,
                 root_rights,
                 Fdflags::empty(),
+                Fdflagsext::empty(),
                 Fd::READ,
                 self.root_inode.clone(),
             )
-            .map_err(|e| format!("Could not create root fd: {}", e))?;
+            .map_err(|e| format!("Could not create root fd: {e}"))?;
         self.preopen_fds.write().unwrap().push(fd);
         Ok(())
     }
@@ -1743,23 +1905,26 @@ impl WasiFs {
                 .create_inode(inodes, kind, true, preopen_name.clone())
                 .map_err(|e| {
                     format!(
-                        "Failed to create inode for preopened dir (name `{}`): WASI error code: {}",
-                        preopen_name, e
+                        "Failed to create inode for preopened dir (name `{preopen_name}`): WASI error code: {e}",
                     )
                 })?;
             let fd_flags = Fd::READ;
             let fd = self
-                .create_fd(rights, rights, Fdflags::empty(), fd_flags, inode.clone())
-                .map_err(|e| format!("Could not open fd for file {:?}: {}", preopen_name, e))?;
+                .create_fd(
+                    rights,
+                    rights,
+                    Fdflags::empty(),
+                    Fdflagsext::empty(),
+                    fd_flags,
+                    inode.clone(),
+                )
+                .map_err(|e| format!("Could not open fd for file {preopen_name:?}: {e}"))?;
             {
                 let mut guard = self.root_inode.write();
                 if let Kind::Root { entries } = guard.deref_mut() {
                     let existing_entry = entries.insert(preopen_name.clone(), inode);
                     if existing_entry.is_some() && !ignore_duplicates {
-                        return Err(format!(
-                            "Found duplicate entry for alias `{}`",
-                            preopen_name
-                        ));
+                        return Err(format!("Found duplicate entry for alias `{preopen_name}`"));
                     }
                 }
             }
@@ -1782,7 +1947,7 @@ impl WasiFs {
             let cur_dir_metadata = self
                 .root_fs
                 .metadata(path)
-                .map_err(|e| format!("Could not get metadata for file {:?}: {}", path, e))?;
+                .map_err(|e| format!("Could not get metadata for file {path:?}: {e}"))?;
 
             let kind = if cur_dir_metadata.is_dir() {
                 Kind::Dir {
@@ -1846,10 +2011,7 @@ impl WasiFs {
                 self.create_inode(inodes, kind, true, path.to_string_lossy().into_owned())
             }
             .map_err(|e| {
-                format!(
-                    "Failed to create inode for preopened dir: WASI error code: {}",
-                    e
-                )
+                format!("Failed to create inode for preopened dir: WASI error code: {e}")
             })?;
             let fd_flags = {
                 let mut fd_flags = 0;
@@ -1866,8 +2028,15 @@ impl WasiFs {
                 fd_flags
             };
             let fd = self
-                .create_fd(rights, rights, Fdflags::empty(), fd_flags, inode.clone())
-                .map_err(|e| format!("Could not open fd for file {:?}: {}", path, e))?;
+                .create_fd(
+                    rights,
+                    rights,
+                    Fdflags::empty(),
+                    Fdflagsext::empty(),
+                    fd_flags,
+                    inode.clone(),
+                )
+                .map_err(|e| format!("Could not open fd for file {path:?}: {e}"))?;
             {
                 let mut guard = self.root_inode.write();
                 if let Kind::Root { entries } = guard.deref_mut() {
@@ -1878,7 +2047,7 @@ impl WasiFs {
                     };
                     let existing_entry = entries.insert(key.clone(), inode);
                     if existing_entry.is_some() && !ignore_duplicates {
-                        return Err(format!("Found duplicate entry for alias `{}`", key));
+                        return Err(format!("Found duplicate entry for alias `{key}`"));
                     }
                 }
             }
@@ -1921,12 +2090,15 @@ impl WasiFs {
             false,
             raw_fd,
             Fd {
-                rights,
-                rights_inheriting: Rights::empty(),
-                flags: fd_flags,
+                inner: FdInner {
+                    rights,
+                    rights_inheriting: Rights::empty(),
+                    flags: fd_flags,
+                    offset: Arc::new(AtomicU64::new(0)),
+                    fd_flags: Fdflagsext::empty(),
+                },
                 // since we're not calling open on this, we don't need open flags
                 open_flags: 0,
-                offset: Arc::new(AtomicU64::new(0)),
                 inode,
                 is_stdio: true,
             },
@@ -2045,7 +2217,7 @@ impl std::fmt::Debug for WasiFs {
 pub fn default_fs_backing() -> Box<dyn virtual_fs::FileSystem + Send + Sync> {
     cfg_if::cfg_if! {
         if #[cfg(feature = "host-fs")] {
-            Box::new(virtual_fs::host_fs::FileSystem::new(Handle::current(), "/").unwrap())
+            Box::new(virtual_fs::host_fs::FileSystem::new(tokio::runtime::Handle::current(), "/").unwrap())
         } else if #[cfg(not(feature = "host-fs"))] {
             Box::<virtual_fs::mem_fs::FileSystem>::default()
         } else {
