@@ -15,12 +15,12 @@ impl JournalEffector {
             })?;
 
         let ret = crate::syscalls::fd_renumber_internal(ctx, ret_fd, fd);
-        if ret != Errno::Success {
+        if !matches!(ret, Ok(Errno::Success)) {
             bail!(
                 "journal restore error: failed renumber file descriptor after epoll create (from={}, to={}) - {}",
                 ret_fd,
                 fd,
-                ret
+                ret.unwrap_or(Errno::Unknown)
             );
         }
 
