@@ -5,12 +5,14 @@ impl JournalEffector {
         ctx: &mut FunctionEnvMut<'_, WasiEnv>,
         original_fd: Fd,
         copied_fd: Fd,
+        cloexec: bool,
     ) -> anyhow::Result<()> {
         Self::save_event(
             ctx,
-            JournalEntry::DuplicateFileDescriptorV1 {
+            JournalEntry::DuplicateFileDescriptorV2 {
                 original_fd,
                 copied_fd,
+                cloexec,
             },
         )
     }
@@ -19,8 +21,9 @@ impl JournalEffector {
         ctx: &mut FunctionEnvMut<'_, WasiEnv>,
         original_fd: Fd,
         copied_fd: Fd,
+        cloexec: bool,
     ) -> anyhow::Result<()> {
-        let ret_fd = crate::syscalls::fd_dup_internal(ctx, original_fd)
+        let ret_fd = crate::syscalls::fd_dup_internal(ctx, original_fd, 0, cloexec)
             .map_err(|err| {
                 anyhow::format_err!(
                     "journal restore error: failed to duplicate file descriptor (original={}, copied={}) - {}",
@@ -30,15 +33,18 @@ impl JournalEffector {
                 )
             })?;
 
-        let ret = crate::syscalls::fd_renumber_internal(ctx, ret_fd, copied_fd);
-        if ret != Errno::Success {
-            bail!(
-                "journal restore error: failed renumber file descriptor after duplicate (from={}, to={}) - {}",
-                ret_fd,
-                copied_fd,
-                ret
-            );
+        if ret_fd != copied_fd {
+            let ret = crate::syscalls::fd_renumber_internal(ctx, ret_fd, copied_fd);
+            if !matches!(ret, Ok(Errno::Success)) {
+                bail!(
+                    "journal restore error: failed renumber file descriptor after duplicate (from={}, to={}) - {}",
+                    ret_fd,
+                    copied_fd,
+                    ret.unwrap_or(Errno::Unknown)
+                );
+            }
         }
+
         Ok(())
     }
 }
