@@ -90,7 +90,9 @@ impl JournalEffector {
         }
 
         let rights = Rights::all_socket();
-        let ret = state.fs.with_fd(rights, rights, new_flags, 0, inode, fd);
+        let ret = state
+            .fs
+            .with_fd(rights, rights, new_flags, Fdflagsext::empty(), 0, inode, fd);
         ret.map_err(|err| {
             anyhow::format_err!(
                 "journal restore error: failed to create remote accepted socket - {}",
