@@ -7,7 +7,8 @@ use std::{
     time::Duration,
 };
 
-use crate::{runtime::module_cache::ModuleHash, WasiProcess, WasiProcessId};
+use crate::{WasiProcess, WasiProcessId};
+use wasmer_types::ModuleHash;
 
 #[derive(Debug, Clone)]
 pub struct WasiControlPlane {
@@ -212,7 +213,7 @@ pub enum ControlPlaneError {
 mod tests {
     use wasmer_wasix_types::wasix::ThreadStartType;
 
-    use crate::os::task::thread::WasiMemoryLayout;
+    use crate::{os::task::thread::WasiMemoryLayout, utils::xxhash_random};
 
     use super::*;
 
@@ -225,7 +226,7 @@ mod tests {
             enable_exponential_cpu_backoff: None,
         });
 
-        let p1 = p.new_process(ModuleHash::random()).unwrap();
+        let p1 = p.new_process(xxhash_random()).unwrap();
         let _t1 = p1
             .new_thread(WasiMemoryLayout::default(), ThreadStartType::MainThread)
             .unwrap();
@@ -234,7 +235,7 @@ mod tests {
             .unwrap();
 
         assert_eq!(
-            p.new_process(ModuleHash::random()).unwrap_err(),
+            p.new_process(xxhash_random()).unwrap_err(),
             ControlPlaneError::TaskLimitReached { max: 2 }
         );
     }
@@ -248,7 +249,7 @@ mod tests {
             enable_exponential_cpu_backoff: None,
         });
 
-        let p1 = p.new_process(ModuleHash::random()).unwrap();
+        let p1 = p.new_process(xxhash_random()).unwrap();
 
         for _ in 0..10 {
             let _thread = p1
@@ -264,7 +265,7 @@ mod tests {
             .unwrap();
 
         assert_eq!(
-            p.new_process(ModuleHash::random()).unwrap_err(),
+            p.new_process(xxhash_random()).unwrap_err(),
             ControlPlaneError::TaskLimitReached { max: 2 }
         );
     }
