@@ -8,11 +8,11 @@ pub use stream::*;
 
 use super::*;
 
-pub(super) use hyper::{service::Service, Uri};
+pub(super) use hyper::Uri;
 pub(super) use std::pin::Pin;
 pub(super) type BoxError = Box<dyn std::error::Error + Send + Sync>;
 pub(super) use std::{
     future::Future,
     task::{Context, Poll},
 };
-pub(super) use tokio::io::{AsyncRead, AsyncWrite, ReadBuf};
+pub(super) use tokio::io::AsyncWrite;
