@@ -3,6 +3,8 @@ use std::{
     time::Instant,
 };
 
+use hyper_util::client::legacy::Client;
+
 use super::{hyper_proxy::HyperProxyConnector, socket_manager::SocketManager};
 
 #[derive(Debug, Clone)]
@@ -11,5 +13,5 @@ pub struct DProxyInstance {
     pub(super) last_used: Arc<Mutex<Instant>>,
     #[allow(unused)]
     pub(super) socket_manager: Arc<SocketManager>,
-    pub(super) client: hyper::Client<HyperProxyConnector>,
+    pub(super) client: Client<HyperProxyConnector, hyper::body::Incoming>,
 }
