@@ -101,7 +101,7 @@ impl crate::runners::Runner for DProxyRunner {
                             let fut = graceful.watch(conn);
                             futs.push(async move {
                                 if let Err(e) = fut.await {
-                                    eprintln!("Error serving connection: {:?}", e);
+                                    eprintln!("Error serving connection: {e:?}");
                                 }
                             });
                         },
