@@ -59,7 +59,7 @@ fn registry_error_message(specifier: &PackageSource) -> String {
         }
         PackageSource::Url(url) => format!("Unable to resolve \"{url}\""),
         PackageSource::Path(path) => {
-            format!("Unable to load \"{}\" from disk", path)
+            format!("Unable to load \"{path}\" from disk")
         }
     }
 }
@@ -421,7 +421,7 @@ mod tests {
         }
 
         fn finish(&self) -> MultiSource {
-            let mut registry = MultiSource::new();
+            let mut registry = MultiSource::default();
             registry.add_source(self.0.clone());
             registry
         }
