@@ -6,6 +6,7 @@ impl<'a, 'c> JournalSyscallPlayer<'a, 'c> {
         &mut self,
         original_fd: u32,
         copied_fd: u32,
+        cloexec: bool,
     ) -> Result<(), WasiRuntimeError> {
         tracing::trace!(%original_fd, %copied_fd, "Replay journal - FdDuplicate");
         self.real_fd.insert(copied_fd);
@@ -19,7 +20,7 @@ impl<'a, 'c> JournalSyscallPlayer<'a, 'c> {
         if self.stderr_fds.contains(&original_fd) {
             self.stderr_fds.insert(copied_fd);
         }
-        JournalEffector::apply_fd_duplicate(&mut self.ctx, original_fd, copied_fd)
+        JournalEffector::apply_fd_duplicate(&mut self.ctx, original_fd, copied_fd, cloexec)
             .map_err(anyhow_err_to_runtime_err)?;
         Ok(())
     }
