@@ -6,6 +6,7 @@ mod fd_dup;
 mod fd_open;
 mod fd_renumber;
 mod fd_seek;
+mod fd_set_fdflags;
 mod fd_set_flags;
 mod fd_set_rights;
 mod fd_set_size;
@@ -21,7 +22,7 @@ mod update_memory;
 
 use crate::journal::JournalEffector;
 use crate::syscalls::anyhow_err_to_runtime_err;
-use crate::syscalls::JournalSyscallPlayer;
+use crate::syscalls::{JournalStdIoWrite, JournalSyscallPlayer};
 use crate::RewindState;
 use crate::WasiRuntimeError;
 use crate::WasiThreadId;
@@ -36,10 +37,10 @@ use wasmer_types::Memory64;
 use wasmer_wasix_types::wasi::Advice;
 use wasmer_wasix_types::wasi::ExitCode;
 use wasmer_wasix_types::wasi::Fd;
-use wasmer_wasix_types::wasi::Fdflags;
 use wasmer_wasix_types::wasi::Filesize;
 use wasmer_wasix_types::wasi::Tty;
 use wasmer_wasix_types::wasi::Whence;
+use wasmer_wasix_types::wasi::{Fdflags, Fdflagsext};
 use wasmer_wasix_types::wasi::{Fstflags, LookupFlags, Timestamp};
 use wasmer_wasix_types::wasi::{Oflags, Rights};
 use wasmer_wasix_types::wasix::ThreadStartType;
