@@ -7,12 +7,19 @@ impl<'a, 'c> JournalSyscallPlayer<'a, 'c> {
     ) {
         tracing::trace!("Replay journal - ClearEthereal");
         self.spawn_threads.clear();
-        self.stdout.clear();
-        self.stderr.clear();
+
+        if let Some(x) = self.stdout.as_mut() {
+            x.clear();
+        }
         self.stdout_fds.clear();
-        self.stderr_fds.clear();
         self.stdout_fds.insert(1 as WasiFd);
+
+        if let Some(x) = self.stderr.as_mut() {
+            x.clear();
+        }
+        self.stderr_fds.clear();
         self.stderr_fds.insert(2 as WasiFd);
+
         differ_ethereal.iter_mut().for_each(|e| e.clear());
         self.staged_differ_memory.clear();
     }
