@@ -108,6 +108,31 @@ impl<'a, 'c> JournalSyscallPlayer<'a, 'c> {
                     fs_rights_base,
                     fs_rights_inheriting,
                     fs_flags,
+                    Fdflagsext::empty(),
+                )?;
+            }
+            JournalEntry::OpenFileDescriptorV2 {
+                fd,
+                dirfd,
+                dirflags,
+                path,
+                o_flags,
+                fs_rights_base,
+                fs_rights_inheriting,
+                fs_flags,
+                fd_flags,
+            } => {
+                self.real_fd.insert(fd);
+                self.action_fd_open(
+                    fd,
+                    dirfd,
+                    dirflags,
+                    path,
+                    o_flags,
+                    fs_rights_base,
+                    fs_rights_inheriting,
+                    fs_flags,
+                    fd_flags,
                 )?;
             }
             JournalEntry::RemoveDirectoryV1 { fd, path } => {
@@ -159,7 +184,7 @@ impl<'a, 'c> JournalSyscallPlayer<'a, 'c> {
                 copied_fd,
             } => {
                 if self.real_fd.contains(&original_fd) {
-                    self.action_fd_dup(original_fd, copied_fd)?;
+                    self.action_fd_dup(original_fd, copied_fd, false)?;
                 } else if let Some(differ_ethereal) = differ_ethereal {
                     tracing::trace!(%original_fd, %copied_fd, "Differ(ether) journal - FdDuplicate");
                     differ_ethereal.push(JournalEntry::DuplicateFileDescriptorV1 {
@@ -167,7 +192,25 @@ impl<'a, 'c> JournalSyscallPlayer<'a, 'c> {
                         copied_fd,
                     });
                 } else {
-                    self.action_fd_dup(original_fd, copied_fd)?;
+                    self.action_fd_dup(original_fd, copied_fd, false)?;
+                }
+            }
+            JournalEntry::DuplicateFileDescriptorV2 {
+                original_fd,
+                copied_fd,
+                cloexec,
+            } => {
+                if self.real_fd.contains(&original_fd) {
+                    self.action_fd_dup(original_fd, copied_fd, cloexec)?;
+                } else if let Some(differ_ethereal) = differ_ethereal {
+                    tracing::trace!(%original_fd, %copied_fd, %cloexec, "Differ(ether) journal - FdDuplicate");
+                    differ_ethereal.push(JournalEntry::DuplicateFileDescriptorV2 {
+                        original_fd,
+                        copied_fd,
+                        cloexec,
+                    });
+                } else {
+                    self.action_fd_dup(original_fd, copied_fd, cloexec)?;
                 }
             }
             JournalEntry::CreateDirectoryV1 { fd, path } => {
@@ -229,6 +272,16 @@ impl<'a, 'c> JournalSyscallPlayer<'a, 'c> {
                     self.action_fd_set_size(fd, st_size)?;
                 }
             }
+            JournalEntry::FileDescriptorSetFdFlagsV1 { fd, flags } => {
+                if self.real_fd.contains(&fd) {
+                    self.action_fd_set_fdflags(fd, flags)?;
+                } else if let Some(differ_ethereal) = differ_ethereal {
+                    tracing::trace!(%fd, ?flags, "Differ(ether) journal - FdSetFdFlags");
+                    differ_ethereal.push(JournalEntry::FileDescriptorSetFdFlagsV1 { fd, flags });
+                } else {
+                    self.action_fd_set_fdflags(fd, flags)?;
+                }
+            }
             JournalEntry::FileDescriptorSetFlagsV1 { fd, flags } => {
                 if self.real_fd.contains(&fd) {
                     self.action_fd_set_flags(fd, flags)?;
@@ -323,13 +376,13 @@ impl<'a, 'c> JournalSyscallPlayer<'a, 'c> {
                 JournalEffector::apply_chdir(&mut self.ctx, &path)
                     .map_err(anyhow_err_to_runtime_err)?;
             }
-            JournalEntry::CreatePipeV1 { fd1, fd2 } => {
+            JournalEntry::CreatePipeV1 { read_fd, write_fd } => {
                 if let Some(differ_ethereal) = differ_ethereal {
-                    tracing::trace!(%fd1, %fd2,  "Differ(ether) journal - CreatePipe");
-                    differ_ethereal.push(JournalEntry::CreatePipeV1 { fd1, fd2 });
+                    tracing::trace!(%read_fd, %write_fd,  "Differ(ether) journal - CreatePipe");
+                    differ_ethereal.push(JournalEntry::CreatePipeV1 { read_fd, write_fd });
                 } else {
-                    tracing::trace!(%fd1, %fd2,  "Replay journal - CreatePipe");
-                    JournalEffector::apply_fd_pipe(&mut self.ctx, fd1, fd2)
+                    tracing::trace!(%read_fd, %write_fd,  "Replay journal - CreatePipe");
+                    JournalEffector::apply_fd_pipe(&mut self.ctx, read_fd, write_fd)
                         .map_err(anyhow_err_to_runtime_err)?;
                 }
             }
@@ -446,6 +499,16 @@ impl<'a, 'c> JournalSyscallPlayer<'a, 'c> {
                         .map_err(anyhow_err_to_runtime_err)?
                 }
             }
+            JournalEntry::SocketPairV1 { fd1, fd2 } => {
+                if let Some(differ_ethereal) = differ_ethereal {
+                    tracing::trace!(%fd1, %fd2, "Differ(ether) journal - SocketPair");
+                    differ_ethereal.push(JournalEntry::SocketPairV1 { fd1, fd2 });
+                } else {
+                    tracing::trace!(%fd1, %fd2, "Replay journal - SocketOpen");
+                    JournalEffector::apply_sock_pair(&mut self.ctx, fd1, fd2)
+                        .map_err(anyhow_err_to_runtime_err)?
+                }
+            }
             JournalEntry::SocketListenV1 { fd, backlog } => {
                 if let Some(differ_ethereal) = differ_ethereal {
                     tracing::trace!(%fd, "Differ(ether) journal - SocketListen");
