@@ -22,7 +22,7 @@ pub fn clock_time_set<M: MemorySize>(
         if env.enable_journal {
             JournalEffector::save_clock_time_set(&mut ctx, clock_id, time).map_err(|err| {
                 tracing::error!("failed to save clock time set event - {}", err);
-                WasiError::Exit(ExitCode::Errno(Errno::Fault))
+                WasiError::Exit(ExitCode::from(Errno::Fault))
             })?;
         }
     }
