@@ -28,7 +28,15 @@ pub fn fd_close(mut ctx: FunctionEnvMut<'_, WasiEnv>, fd: WasiFd) -> Result<Errn
             return Ok(Errno::Success);
         }
     }
-
+    // HACK: we use tokio files to back WASI file handles. Since tokio
+    // does writes in the background, it may miss writes if the file is
+    // closed without flushing first. Hence, we flush once here.
+    match __asyncify_light(env, None, state.fs.flush(fd))? {
+        Ok(_) | Err(Errno::Isdir) | Err(Errno::Io) | Err(Errno::Access) => {}
+        Err(e) => {
+            return Ok(e);
+        }
+    }
     wasi_try_ok!(state.fs.close_fd(fd));
 
     #[cfg(feature = "journal")]
