@@ -79,7 +79,11 @@ pub(crate) fn path_filestat_set_times_internal(
     let (memory, mut state, inodes) = unsafe { env.get_memory_and_wasi_state_and_inodes(&ctx, 0) };
     let fd_entry = state.fs.get_fd(fd)?;
     let fd_inode = fd_entry.inode;
-    if !fd_entry.rights.contains(Rights::PATH_FILESTAT_SET_TIMES) {
+    if !fd_entry
+        .inner
+        .rights
+        .contains(Rights::PATH_FILESTAT_SET_TIMES)
+    {
         return Err(Errno::Access);
     }
     if (fst_flags.contains(Fstflags::SET_ATIM) && fst_flags.contains(Fstflags::SET_ATIM_NOW))
