@@ -14,7 +14,7 @@ pub fn chdir<M: MemorySize>(
     let path = unsafe { get_input_str_ok!(&memory, path, path_len) };
     Span::current().record("path", path.as_str());
 
-    wasi_try_ok!(chdir_internal(&mut ctx, &path,));
+    wasi_try_ok!(chdir_internal(ctx.data(), &path));
     let env = ctx.data();
 
     #[cfg(feature = "journal")]
@@ -28,9 +28,8 @@ pub fn chdir<M: MemorySize>(
     Ok(Errno::Success)
 }
 
-pub fn chdir_internal(ctx: &mut FunctionEnvMut<'_, WasiEnv>, path: &str) -> Result<(), Errno> {
-    let env = ctx.data();
-    let (memory, mut state) = unsafe { env.get_memory_and_wasi_state(ctx, 0) };
+pub fn chdir_internal(env: &WasiEnv, path: &str) -> Result<(), Errno> {
+    let state = &env.state;
 
     // Check if the directory exists
     if state.fs.root_fs.read_dir(Path::new(path)).is_err() {
