@@ -11,7 +11,7 @@ pub fn port_route_clear(mut ctx: FunctionEnvMut<'_, WasiEnv>) -> Result<Errno, W
     if ctx.data().enable_journal {
         JournalEffector::save_port_route_clear(&mut ctx).map_err(|err| {
             tracing::error!("failed to save port_route_clear event - {}", err);
-            WasiError::Exit(ExitCode::Errno(Errno::Fault))
+            WasiError::Exit(ExitCode::from(Errno::Fault))
         })?;
     }
 
