@@ -29,8 +29,10 @@ pub fn sock_send<M: MemorySize>(
 
     let env = ctx.data();
     let fd_entry = wasi_try_ok!(env.state.fs.get_fd(fd));
+    let enable_journal = env.enable_journal;
     let guard = fd_entry.inode.read();
-    let use_write = matches!(guard.deref(), Kind::Pipe { .. });
+    // We need this hack because we use a pipe to back socket pairs
+    let use_write = matches!(guard.deref(), Kind::DuplexPipe { .. });
     drop(guard);
 
     let bytes_written = if use_write {
@@ -39,11 +41,11 @@ pub fn sock_send<M: MemorySize>(
             let inodes = state.inodes.clone();
 
             let fd_entry = wasi_try_ok!(state.fs.get_fd(fd));
-            fd_entry.offset.load(Ordering::Acquire) as usize
+            fd_entry.inner.offset.load(Ordering::Acquire) as usize
         };
 
         wasi_try_ok!(fd_write_internal::<M>(
-            &ctx,
+            &mut ctx,
             fd,
             FdWriteSource::Iovs {
                 iovs: si_data,
@@ -51,7 +53,7 @@ pub fn sock_send<M: MemorySize>(
             },
             offset as u64,
             true,
-            env.enable_journal,
+            enable_journal
         )?)
     } else {
         wasi_try_ok!(sock_send_internal::<M>(
@@ -100,7 +102,7 @@ pub(crate) fn sock_send_internal<M: MemorySize>(
         sock,
         Rights::SOCK_SEND,
         |socket, fd| async move {
-            let nonblocking = fd.flags.contains(Fdflags::NONBLOCK);
+            let nonblocking = fd.inner.flags.contains(Fdflags::NONBLOCK);
             let timeout = socket
                 .opt_time(TimeType::WriteTimeout)
                 .ok()
