@@ -75,7 +75,7 @@ pub(crate) fn sock_send_file_internal(
         count -= sub_count;
 
         let fd_entry = wasi_try_ok_ok!(state.fs.get_fd(in_fd));
-        let fd_flags = fd_entry.flags;
+        let fd_flags = fd_entry.inner.flags;
 
         let data = {
             match in_fd {
@@ -95,12 +95,12 @@ pub(crate) fn sock_send_file_internal(
                 }
                 __WASI_STDOUT_FILENO | __WASI_STDERR_FILENO => return Ok(Err(Errno::Inval)),
                 _ => {
-                    if !fd_entry.rights.contains(Rights::FD_READ) {
+                    if !fd_entry.inner.rights.contains(Rights::FD_READ) {
                         // TODO: figure out the error to return when lacking rights
                         return Ok(Err(Errno::Access));
                     }
 
-                    let offset = fd_entry.offset.load(Ordering::Acquire) as usize;
+                    let offset = fd_entry.inner.offset.load(Ordering::Acquire) as usize;
                     let inode = fd_entry.inode;
                     let data = {
                         let mut guard = inode.write();
@@ -159,7 +159,20 @@ pub(crate) fn sock_send_file_internal(
                                 env = ctx.data();
                                 data
                             }
-                            Kind::Pipe { ref mut pipe, .. } => {
+                            Kind::PipeRx { ref mut rx } => {
+                                let data = wasi_try_ok_ok!(__asyncify(ctx, None, async move {
+                                    // TODO: optimize with MaybeUninit
+                                    let mut buf = vec![0u8; sub_count as usize];
+                                    let amt = virtual_fs::AsyncReadExt::read(rx, &mut buf[..])
+                                        .await
+                                        .map_err(map_io_err)?;
+                                    buf.truncate(amt);
+                                    Ok(buf)
+                                })?);
+                                env = ctx.data();
+                                data
+                            }
+                            Kind::DuplexPipe { ref mut pipe } => {
                                 let data = wasi_try_ok_ok!(__asyncify(ctx, None, async move {
                                     // TODO: optimize with MaybeUninit
                                     let mut buf = vec![0u8; sub_count as usize];
@@ -172,15 +185,14 @@ pub(crate) fn sock_send_file_internal(
                                 env = ctx.data();
                                 data
                             }
-                            Kind::Epoll { .. } => {
+                            Kind::PipeTx { .. }
+                            | Kind::Epoll { .. }
+                            | Kind::EventNotifications { .. } => {
                                 return Ok(Err(Errno::Inval));
                             }
                             Kind::Dir { .. } | Kind::Root { .. } => {
                                 return Ok(Err(Errno::Isdir));
                             }
-                            Kind::EventNotifications { .. } => {
-                                return Ok(Err(Errno::Inval));
-                            }
                             Kind::Symlink { .. } => unimplemented!("Symlinks in wasi::fd_read"),
                             Kind::Buffer { buffer } => {
                                 // TODO: optimize with MaybeUninit
