@@ -37,10 +37,7 @@ pub(super) fn thread_join_internal<M: MemorySize + 'static>(
             other_thread
                 .join()
                 .await
-                .map_err(|err| {
-                    err.as_exit_code()
-                        .unwrap_or(ExitCode::Errno(Errno::Unknown))
-                })
+                .map_err(|err| err.as_exit_code().unwrap_or(ExitCode::from(Errno::Unknown)))
                 .unwrap_or_else(|a| a)
                 .raw()
         })?;
