@@ -11,12 +11,22 @@ pub use self::{dummy_waker::WasiDummyWaker, thread_parker::WasiParkingLot};
 pub(crate) use owned_mutex_guard::{
     read_owned, write_owned, OwnedRwLockReadGuard, OwnedRwLockWriteGuard,
 };
+use rand::RngCore;
+use wasmer_types::ModuleHash;
 
 use std::collections::BTreeSet;
 
 use wasmer::Module;
 use wasmer_wasix_types::wasi::Errno;
 
+/// Creates a random xxhash for the module
+pub fn xxhash_random() -> ModuleHash {
+    let mut rand = rand::thread_rng();
+    let mut key = [0u8; 8];
+    rand.fill_bytes(&mut key);
+    ModuleHash::xxhash_from_bytes(key)
+}
+
 /// Check if a provided module is compiled for some version of WASI.
 /// Use [`get_wasi_version`] to find out which version of WASI the module is.
 pub fn is_wasi_module(module: &Module) -> bool {
