@@ -6,11 +6,11 @@ bash build.sh
 status=0
 
 # Define skip list as an array
-SKIP_LIST=("fs_create_dir-existing-directory.wasm")
+SKIP_LIST=()
 
 # List and process .foo files
 for file in *.wasm; do
-    if [[ " ${SKIP_LIST[@]} " =~ " ${file} " ]]; then
+    if [[ " ${SKIP_LIST[0]+${SKIP_LIST[@]}} " =~ " ${file} " ]]; then
         echo "Skipping $file"
     else
         echo "Testing $file"
